#ifndef ACTIONMANAGER_H
#define ACTIONMANAGER_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include "action.h"

#include <qobject.h>
#include <qdatetime.h>
#include <qptrlist.h>

namespace SubtitleComposer
{
	class ActionManager : public QObject
	{
		Q_OBJECT

		public:

			ActionManager();
			~ActionManager();

			bool hasRedo() const;
			bool hasUndo() const;

			QString redoDescription() const;
			QString undoDescription() const;

			void execAndStore( Action* action );

			void popUndo();

		public slots:

			void redo();
			void undo();
			void clearHistory();

		signals:

			void actionStored();
			void actionRemoved();
			void actionRedone();
			void actionUndone();
			void historyCleared();
			void stateChanged();

		private:

			ActionManager( const ActionManager& undoManager );
			ActionManager& operator=( const ActionManager& undoManager );

			mutable QPtrList<Action> m_undoStack;
			mutable QPtrList<Action> m_redoStack;

			int m_compressionThreshold;
			QTime m_time;
	};
}

#endif
