 
/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "audiolevels.h"

#include <vector>
#include <cmath> // TODO remove when unneeded

#include <qfile.h>
#include <qtextstream.h>

using namespace SubtitleComposer;

AudioLevels::AudioLevels():
	m_channelsCount( 0 ),
	m_samplesInterval( 50 ),
	m_samplesCount( 0 ),
	m_samplesData( 0 ),
	m_lastDataIndex( -1 ),
	m_mediaUrl()
{
}

AudioLevels::~AudioLevels()
{
	delete [] m_samplesData;
}

bool AudioLevels::load( const QString& filePath )
{
	if ( m_samplesData )
		return false;

	QFile file( filePath );
	if ( ! file.open( IO_ReadOnly ) )
		return false;

	QTextStream textStream( &file );

	if ( QString( "audiolevels" ) != textStream.readLine() )
	{
		file.close();
		return false;
	}

	m_mediaUrl = KURL( textStream.readLine() );

	file.readBlock( (char*)&m_channelsCount, sizeof(m_channelsCount) );

	Q_INT64 duration;
	file.readBlock( (char*)&duration, sizeof(duration) );

	m_samplesCount = 0;
	for( unsigned channel = 0; channel< m_channelsCount; ++channel )
	{
		std::vector<double>::size_type samplesCount;
		file.readBlock( (char*)&samplesCount, sizeof(samplesCount) );

		if ( ! m_samplesCount )
		{
			m_samplesCount = samplesCount;
			m_samplesData = new double[m_samplesCount*m_channelsCount];
			m_lastDataIndex = m_samplesCount*m_channelsCount - 1;
		}
		else if ( m_samplesCount != samplesCount )
		{
			reset();
			file.close();
			return false;
		}

		file.readBlock( (char*)&m_samplesData[m_samplesCount*channel], sizeof(double)*m_samplesCount );
	}

	file.close();

 	m_samplesInterval = (int)(duration/1000000.0 / m_samplesCount);

	if ( ! m_samplesInterval )
	{
		reset();
		return false;
	}

	return true;
}

bool AudioLevels::loadFromMedia( const QString& /*mediaPath*/, unsigned /*streamIndex*/ )
{
	// STUB IMPLEMENTATION just to get something we can use

	m_channelsCount = 1;
	m_samplesCount = 30000;
	m_samplesData = new double[m_samplesCount*m_channelsCount];
	m_lastDataIndex = m_samplesCount*m_channelsCount - 1;

	double degrees = 0;
	for ( int idx = 0; idx < 30000; ++idx )
	{
		m_samplesData[idx] = sin( degrees*M_PI/180 )/100;
		degrees += 10.0;
	}
	m_samplesInterval = 100;

	return true;

	return false; // TODO stub implementation
}

bool AudioLevels::save( const QString& filePath, bool overwrite ) const
{
	if ( ! overwrite && QFile::exists( filePath ) )
		return false;

	QFile file( filePath );
	if ( ! file.open( IO_WriteOnly|IO_Truncate ) )
		return false;

	QString audiolevels( "audiolevels\n" );
	file.writeBlock( audiolevels.ascii(), audiolevels.length() );

	QString url = m_mediaUrl.url() + "\n";
	file.writeBlock( url.ascii(), url.length() );

	file.writeBlock( (const char*)&m_channelsCount, sizeof(m_channelsCount) );

	Q_INT64 duration = (Q_INT64)m_samplesInterval*(Q_INT64)m_samplesCount*1000000;
	file.writeBlock( (const char*)&duration, sizeof(duration) );

	for( unsigned channel = 0; channel< m_channelsCount; ++channel )
	{
		std::vector<double>::size_type samplesCount = m_samplesCount;
		file.writeBlock( (const char*)&samplesCount, sizeof(samplesCount) );

		file.writeBlock( (const char*)&m_samplesData[m_samplesCount*channel], sizeof(double)*m_samplesCount );
	}

	file.close();

	return true;
}

void AudioLevels::reset()
{
	m_channelsCount = 0;
	m_samplesInterval = 50;
	m_samplesCount = 0;
	delete [] m_samplesData;
	m_samplesData = 0;
	m_lastDataIndex = -1;
	m_mediaUrl = KURL();
}
