#ifndef SUBTITLEACTIONS_H
#define SUBTITLEACTIONS_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include "ranges.h"
#include "subtitle.h"

#include <qstring.h>

namespace SubtitleComposer
{
	class CompositeAction;

	class SubtitleAction : public Action
	{
		public:

			typedef enum {
				None,
				Primary,
				Secondary,
				Both
			} DirtyMode;

			SubtitleAction( Subtitle& subtitle, DirtyMode dirtyMode, const QString& description=QString::null );
			virtual ~SubtitleAction();

			void setLineSubtitle( SubtitleLine* line );
			void clearLineSubtitle( SubtitleLine* line );

			inline static unsigned getLastIndex( const Subtitle& subtitle, unsigned lastIndex )
			{
				return lastIndex == Range::MaxIndex ? subtitle.linesCount()-1 : lastIndex;
			}

			inline static unsigned getInsertionIndex( const Subtitle& subtitle, unsigned lastIndex )
			{
				return lastIndex == Range::MaxIndex ? subtitle.linesCount() : lastIndex;
			}

		protected:

			virtual void _preRedo();
			virtual void _preUndo();

		protected:

			Subtitle& m_subtitle;
			DirtyMode m_dirtyMode;
	};

	class SetFramesPerSecondAction : public SubtitleAction
	{
		public:

			SetFramesPerSecondAction( Subtitle& subtitle, double framesPerSecond );
			virtual ~SetFramesPerSecondAction();

		protected:

			virtual void _redo();
			virtual void _undo();

			virtual void _emitRedoSignals();
			virtual void _emitUndoSignals();

		private:

			double m_framesPerSecond;
	};

	class InsertLinesAction : public SubtitleAction
	{
		public:

			InsertLinesAction( Subtitle& subtitle, unsigned insertIndex, const QPtrList<SubtitleLine>& lines );
			virtual ~InsertLinesAction();

		protected:

			virtual void _redo();
			virtual void _undo();

			virtual void _emitRedoSignals();
			virtual void _emitUndoSignals();

		private:

			unsigned m_insertIndex;
			unsigned m_lastIndex;
			QPtrList<SubtitleLine> m_lines;
	};

	class InsertPartialLineAction : public SubtitleAction
	{
		public:

			InsertPartialLineAction( Subtitle& subtitle, unsigned insertIndex, const SubtitleLine* line, bool primary );
			virtual ~InsertPartialLineAction();

		protected:

			virtual void _redo();
			virtual void _undo();

			virtual void _emitRedoSignals();
			virtual void _emitUndoSignals();

		private:

			unsigned m_insertIndex;
			SubtitleLine m_line;
	};


	class RemoveLinesAction : public SubtitleAction
	{
		public:

			RemoveLinesAction( Subtitle& subtitle, unsigned firstIndex, unsigned lastIndex );
			virtual ~RemoveLinesAction();

		protected:

			virtual void _redo();
			virtual void _undo();

			virtual void _emitRedoSignals();
			virtual void _emitUndoSignals();

		private:

			unsigned m_firstIndex;
			unsigned m_lastIndex;
			QPtrList<SubtitleLine> m_lines;
	};


	class MoveLineAction : public SubtitleAction
	{
		public:

			MoveLineAction( Subtitle& subtitle, unsigned fromIndex, unsigned toIndex );
			virtual ~MoveLineAction();

		protected:

			virtual bool mergeWithPrevious( Action* prevAction );

			virtual void _redo();
			virtual void _undo();

			virtual void _emitRedoSignals();
			virtual void _emitUndoSignals();

		private:

			unsigned m_fromIndex;
			unsigned m_toIndex;
	};

	class SwapLinesTextsAction : public SubtitleAction
	{
		public:

			SwapLinesTextsAction( Subtitle& subtitle, const RangeList& ranges );
			virtual ~SwapLinesTextsAction();

		protected:

			virtual void _redo();
			virtual void _undo();

			virtual void _emitRedoSignals();
			virtual void _emitUndoSignals();

		private:

			RangeList m_ranges;
	};

}

#endif
