#ifndef FORMATSMANAGER_H
#define FORMATSMANAGER_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include "inputformat.h"
#include "outputformat.h"

#include <qptrlist.h>

namespace SubtitleComposer
{
	class FormatsManager
	{
		public:

			static QStringList inputFormatNames();
			static QStringList outputFormatNames();

			static bool hasInputFormat( const QString& name );
			static bool hasOutputFormat( const QString& name );

			static const InputFormat* inputFormat( const QString& name );
			static const OutputFormat* outputFormat( const QString& name );

			static bool readSubtitle( Subtitle& subtitle, bool primary, const QString& path, QTextCodec* codec, Format::NewLine* newLine=0, QString* format=0 );
			static bool writeSubtitle( const Subtitle& subtitle, bool primary, const QString& path, QTextCodec* codec, Format::NewLine newLine, const QString& format, bool overwrite );

		protected:

			static void loadFormats( bool force=false );

			static QPtrList<InputFormat> s_inputFormats;
			static QPtrList<OutputFormat> s_outputFormats;
			static bool s_loaded;
	};
}

#endif
