#ifndef INPUTFORMAT_H
#define INPUTFORMAT_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include "format.h"
#include <subtitle.h>
#include <subtitleiterator.h>

#include <qstringlist.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtextcodec.h>
#include <kdebug.h>

namespace SubtitleComposer
{
	class InputFormat : public Format
	{
		public:

			~InputFormat() {}

			bool readSubtitle( Subtitle& subtitle, NewLine* newLine, bool primary, const QString& path, QTextCodec* codec, bool ignoreExtension ) const
			{
				if ( ! ignoreExtension )
				{
					QString extension( QFileInfo( path ).extension( false ) );

					if ( ! knowsExtension( extension ) )
						return false;
				}

				QFile file( path );
				if ( ! file.open( IO_ReadOnly ) )
				{
					kdDebug() << "couldn't open input file " << file.name() << endl;
					return false;
				}

				QString data;

				QTextDecoder* decoder = codec->makeDecoder();
				char* dataChunk = new char[262144]; // 256KB
				int read = file.readBlock( dataChunk, 262144 );
				while ( read != -1 && read != 0 )
				{
					data += decoder->toUnicode( dataChunk, read );
					read = file.readBlock( dataChunk, 262144 );
				}

				delete [] dataChunk;
				delete decoder;
				file.close();

				if ( newLine )
				{
					if ( data.find( "\r\n" ) != -1 )
						*newLine = Windows;
					else if ( data.find( "\r" ) != -1 )
						*newLine = Macintosh;
					else if ( data.find( "\n" ) != -1 )
						*newLine = UNIX;
					else
						*newLine = CurrentOS;
				}

				data.replace( "\r\n", "\n" );
				data.replace( "\r", "\n" );

				if ( primary )
				{
					// Initialized to the default value, can be overrided later by each format
					subtitle.setFramesPerSecond( Subtitle::defaultFramesPerSecond() );

					setFormatData( subtitle, 0 );
					for ( SubtitleIterator it( subtitle ); it.current(); ++it )
						setFormatData( it.current(), 0 );
				}

				return parseSubtitles( subtitle, primary, data );
			}

			virtual bool parseSubtitles( Subtitle& subtitle, bool primary, const QString& data ) const = 0;

		protected:

			InputFormat( const QString& name, const QStringList& extensions ):
				Format( name, extensions ) {}

			/// use ony in the context of parseSubtitles
			SubtitleLine* setLine( Subtitle& subtitle, unsigned index, bool primary, const QString& text, const Time& showTime, const Time& hideTime, int styleFlags ) const
			{
				SubtitleLine* line;

				if ( primary )
				{
					if ( index < subtitle.linesCount() )
					{
						line = subtitle.line( index );
						line->setPrimaryText( text );
						line->setTimes( showTime, hideTime );
						line->setStyleFlags( styleFlags );
					}
					else
					{
						line = new SubtitleLine( text, QString::null, showTime, hideTime, styleFlags );
						subtitle.insertLine( line );
					}
				}
				else
				{
					if ( index < subtitle.linesCount() )
					{
						line = subtitle.line( index );
						line->setSecondaryText( text );
					}
					else
					{
						line = new SubtitleLine( QString::null, text, showTime, hideTime, styleFlags );
						subtitle.insertLine( line );
					}
				}

				return line;
			}

	};
}

#endif
