#ifndef SUBVIEWER2INPUTFORMAT_H
#define SUBVIEWER2INPUTFORMAT_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <inputformat.h>
#include <qregexp.h>

namespace SubtitleComposer
{
	class SubViewer2InputFormat : public InputFormat
	{
		friend class FormatsManager;

		public:

			~SubViewer2InputFormat() {}

			virtual bool parseSubtitles( Subtitle& subtitle, bool primary, const QString& data ) const
			{
				unsigned readLines = 0;

				for ( int offset=0; m_regExp.search( data, offset ) != -1; offset=m_regExp.pos()+m_regExp.matchedLength() )
				{
					Time showTime(
						m_regExp.cap( 1 ).toInt(),
						m_regExp.cap( 2 ).toInt(),
						m_regExp.cap( 3 ).toInt(),
						m_regExp.cap( 4 ).toInt() * 10
					);

					Time hideTime(
						m_regExp.cap( 5 ).toInt(),
						m_regExp.cap( 6 ).toInt(),
						m_regExp.cap( 7 ).toInt(),
						m_regExp.cap( 8 ).toInt() * 10
					);

					QString text( m_regExp.cap( 9 ).replace( "[br]", "\n" ).stripWhiteSpace() );

					setLine( subtitle, readLines, primary, text, showTime, hideTime, 0 );

					readLines++;
				}

				return readLines > 0;
			}

		protected:

			SubViewer2InputFormat():
				InputFormat( "SubViewer 2.0", "sub" ),
				m_regExp( "([0-2][0-9]):([0-5][0-9]):([0-5][0-9])\\.([0-9][0-9]),"
						  "([0-2][0-9]):([0-5][0-9]):([0-5][0-9])\\.([0-9][0-9])\n"
						  "([^\n]*)\n\n" )
			{
			}

			mutable QRegExp m_regExp;
	};
}

#endif

