#ifndef SUBVIEWER2OUTPUTFORMAT_H
#define SUBVIEWER2OUTPUTFORMAT_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <outputformat.h>
#include <qstring.h>

namespace SubtitleComposer
{
	class SubViewer2OutputFormat : public OutputFormat
	{
		friend class FormatsManager;

		public:

			~SubViewer2OutputFormat() {}

			virtual QString dumpSubtitles( const Subtitle& subtitle, bool primary ) const
			{
				QString ret( "[INFORMATION]\n[TITLE]\n[AUTHOR]\n[SOURCE]\n[PRG]\n[FILEPATH]\n[DELAY]0\n[CD TRACK]0\n"
							 "[COMMENT]\n[END INFORMATION]\n[SUBTITLE]\n[COLF]&HFFFFFF,[STYLE]bd,[SIZE]24,[FONT]Tahoma\n" );

				for ( SubtitleIterator it( subtitle ); it.current(); ++it )
				{
					const SubtitleLine* line = it.current();

					Time showTime = line->showTime();
					Time hideTime = line->hideTime();
					ret += m_builder.sprintf(
						"%02d:%02d:%02d.%02d,%02d:%02d:%02d.%02d\n",
						showTime.hours(), showTime.minutes(), showTime.seconds(), (int)((showTime.mseconds()+5)/10),
						hideTime.hours(), hideTime.minutes(), hideTime.seconds(), (int)((hideTime.mseconds()+5)/10)
					);

					ret += (primary ? line->primaryText() : line->secondaryText()).replace( "\n", "[br]" );

					ret += "\n\n";
				}

				return ret;
			}

		protected:

			SubViewer2OutputFormat():
				OutputFormat( "SubViewer 2.0", "sub" )
			{
			}

			mutable QString m_builder;
	};
}

#endif
