/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "generalconfigwidget.h"
#include "generalconfigdefs.h"
#include <application.h>

#include <qtextcodec.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qlabel.h>
#include <kcombobox.h>
#include <knuminput.h>

#include <klocale.h>
#include <ksconfig.h>

using namespace SubtitleComposer;

GeneralConfigWidget::GeneralConfigWidget( const AppConfigGroup& configGroup, QWidget* parent ):
	AppConfigGroupWidget( configGroup, parent )
{
	QGroupBox* generalGroupBox = createGroupBox( i18n( "General" ) );

// 	QLabel* spellCheckDictLabel = new QLabel( generalGroupBox );
// 	spellCheckDictLabel->setText( i18n( "Spell check dictionary" ) );
//
// 	m_spellCheckDictComboBox = new KComboBox( generalGroupBox );
// 	KSpellConfig().fillDicts( m_spellCheckDictComboBox, &m_spellDictionaries );

	QLabel* defaultEncodingLabel = new QLabel( generalGroupBox );
	defaultEncodingLabel->setText( i18n( "Default encoding for opening subtitles" ) );

	m_defaultEncodingComboBox = new KComboBox( generalGroupBox );
	m_defaultEncodingComboBox->insertStringList( app()->availableEncodingNames() );

	QLabel* relativeSeekPositionLabel = new QLabel( generalGroupBox );
	relativeSeekPositionLabel->setText( i18n( "On line double click, jump to show time minus..." ) );

	m_relativeSeekPositionSpinBox = new KIntNumInput( generalGroupBox );
	m_relativeSeekPositionSpinBox->setMinValue( 0 );
	m_relativeSeekPositionSpinBox->setMaxValue( 10000 );
	m_relativeSeekPositionSpinBox->setSuffix( i18n( " msecs" ) );

	m_autoLoadVideoCheckBox = new QCheckBox( generalGroupBox );
	m_autoLoadVideoCheckBox->setText( i18n( "Automatically load video file when opening subtitle" ) );


	QGroupBox* quickActionsGroupBox = createGroupBox( i18n( "Quick actions" ) );

	QLabel* shiftMillisecondsLabel = new QLabel( quickActionsGroupBox );
	shiftMillisecondsLabel->setText( i18n( "Time shift for selected lines" ) );

	m_shiftMsecsSpinBox = new KIntNumInput( quickActionsGroupBox );
	m_shiftMsecsSpinBox->setMinValue( 1 );
	m_shiftMsecsSpinBox->setMaxValue( 1000 );
	m_shiftMsecsSpinBox->setSuffix( i18n( " msecs" ) );

	QLabel* setTimeCompMsecsLabel = new QLabel( quickActionsGroupBox );
	setTimeCompMsecsLabel->setText( i18n( "Time compensation for captured video times" ) );

	m_videoPosCompMsecsSpinBox = new KIntNumInput( quickActionsGroupBox );
	m_videoPosCompMsecsSpinBox->setMinValue( 1 );
	m_videoPosCompMsecsSpinBox->setMaxValue( 1000 );
	m_videoPosCompMsecsSpinBox->setSuffix( i18n( " msecs" ) );


	QGridLayout* generalLayout = createGridLayout( generalGroupBox );
// 	generalLayout->addWidget( spellCheckDictLabel, 0, 0 );
// 	generalLayout->addWidget( m_spellCheckDictComboBox, 0, 1 );
	generalLayout->addWidget( defaultEncodingLabel, 1, 0 );
	generalLayout->addWidget( m_defaultEncodingComboBox, 1, 1 );
	generalLayout->addWidget( relativeSeekPositionLabel, 2, 0 );
	generalLayout->addWidget( m_relativeSeekPositionSpinBox, 2, 1 );
	generalLayout->addWidget( m_autoLoadVideoCheckBox, 3, 0 );

	QGridLayout* quickActionsLayout = createGridLayout( quickActionsGroupBox );
	quickActionsLayout->addWidget( shiftMillisecondsLabel, 0, 0 );
	quickActionsLayout->addWidget( m_shiftMsecsSpinBox, 0, 1 );
	quickActionsLayout->addWidget( setTimeCompMsecsLabel, 1, 0 );
	quickActionsLayout->addWidget( m_videoPosCompMsecsSpinBox, 1, 1 );

// 	connect( m_spellCheckDictComboBox, SIGNAL(activated(int)), this, SIGNAL(settingsChanged()) );
	connect( m_defaultEncodingComboBox, SIGNAL(activated(int)), this, SIGNAL(settingsChanged()) );
	connect( m_relativeSeekPositionSpinBox, SIGNAL(valueChanged(int)), this, SIGNAL(settingsChanged()) );
	connect( m_autoLoadVideoCheckBox, SIGNAL(toggled(bool)), this, SIGNAL(settingsChanged()) );
	connect( m_shiftMsecsSpinBox, SIGNAL(valueChanged(int)), this, SIGNAL(settingsChanged()) );
	connect( m_videoPosCompMsecsSpinBox, SIGNAL(valueChanged(int)), this, SIGNAL(settingsChanged()) );

	setControlsFromConfig();
}

GeneralConfigWidget::~GeneralConfigWidget()
{
}

void GeneralConfigWidget::setConfigFromControls()
{
// 	m_configGroup.setOption( OPT_SPELL_CHECK_DICT, m_spellDictionaries[m_spellCheckDictComboBox->currentItem()] );
	m_configGroup.setOption( OPT_DEFAULT_SUBS_ENCODING, m_defaultEncodingComboBox->currentText() );
	m_configGroup.setOption( OPT_JUMP_TO_POSITION, m_relativeSeekPositionSpinBox->value() );
	m_configGroup.setOption( OPT_AUTO_VIDEO_LOAD, m_autoLoadVideoCheckBox->isChecked() );
	m_configGroup.setOption( OPT_SHIFT_SELECTED_LINES_MSECS, m_shiftMsecsSpinBox->value() );
	m_configGroup.setOption( OPT_VIDEO_POS_COMP_MSECS, m_videoPosCompMsecsSpinBox->value() );
}

void GeneralConfigWidget::setControlsFromConfig()
{
// 	m_spellCheckDictComboBox->setCurrentItem( m_spellDictionaries.findIndex( m_configGroup.option( OPT_SPELL_CHECK_DICT ) ) );
	m_defaultEncodingComboBox->setCurrentItem( m_configGroup.option( OPT_DEFAULT_SUBS_ENCODING ) );
	m_relativeSeekPositionSpinBox->setValue( m_configGroup.optionAsInt( OPT_JUMP_TO_POSITION ) );
	m_autoLoadVideoCheckBox->setChecked( m_configGroup.optionAsBool( OPT_AUTO_VIDEO_LOAD ) );
	m_shiftMsecsSpinBox->setValue( m_configGroup.optionAsInt( OPT_SHIFT_SELECTED_LINES_MSECS ) );
	m_videoPosCompMsecsSpinBox->setValue( m_configGroup.optionAsInt( OPT_VIDEO_POS_COMP_MSECS ) );
}

#include "generalconfigwidget.moc"
