#ifndef CONFIGDIALOG_H
#define CONFIGDIALOG_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <appconfig.h>

#include <kdialogbase.h>

namespace SubtitleComposer
{
	class AppConfigGroupWidget;

	class ConfigDialog : public KDialogBase
	{
		Q_OBJECT

		public:

			typedef enum { General=0, Errors, Player, FirstBackend } Pages;


			ConfigDialog( const AppConfig& config, QWidget* parent=0 );
			virtual ~ConfigDialog();

			void setConfig( const AppConfig& config );
			const AppConfig& config() const;

		public slots:

			void show();
			void hide();

		signals:

			void accepted();

		protected slots:

			void acceptConfig();
			void acceptConfigAndClose();
			void rejectConfig();
			void rejectConfigAndClose();

			void setControlsFromConfig();
			void setControlsFromConfig( unsigned pageIndex );

			void setConfigFromControls();
			void setConfigFromControls( unsigned pageIndex );

			void setActivePageControlsFromDefaults();
			void setControlsFromDefaults( unsigned pageIndex );

			void enableApply();

			void onOptionChanged( const QString& groupName, const QString& optionName, const QString& value );

		protected:

			void addPage( QWidget* page, const QString& itemName, const QPixmap& pixmap, const QString& header );
			void addPage( QWidget* page, const QString& itemName, const QString& pixmapName, const QString& header );

		protected:

			AppConfig m_acceptedConfig;
			AppConfig m_config;

			QPtrList<AppConfigGroupWidget> m_configWidgets;
	};
}

#endif
