/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "actiondialog.h"

#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qlayout.h>

#include <kapplication.h>
#include <kaboutdata.h>

using namespace SubtitleComposer;

ActionDialog::ActionDialog( const QString& title, QWidget* parent ):
	KDialogBase( parent, 0, true, title, KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok )
{
	m_mainWidget = new QWidget( this );
	m_mainLayout = new QGridLayout( m_mainWidget );
	m_mainLayout->setAlignment( Qt::AlignTop );
	m_mainLayout->setMargin( 0 );
	m_mainLayout->setSpacing( 5 );

	setMainWidget( m_mainWidget );

	// attempt at showing the full action name on the title...
	// it would work better if we could get the fontMetrics used by kwin
	m_mainWidget->setMinimumWidth( fontMetrics().width( title + " - " + kapp->aboutData()->appName() ) + 120 );
}

void ActionDialog::show()
{
	KDialogBase::show();
	adjustSize();
}


QGroupBox* ActionDialog::createGroupBox( const QString& title, bool addToLayout )
{
	QButtonGroup* buttonGroup = new QButtonGroup( m_mainWidget );
	buttonGroup->setTitle( title );
	buttonGroup->setColumnLayout( 0, Qt::Vertical );
	buttonGroup->layout()->setSpacing( 5 );

	if ( addToLayout )
		m_mainLayout->addWidget( buttonGroup, m_mainLayout->numRows(), 0 );

	return buttonGroup;
}

QGridLayout* ActionDialog::createLayout( QGroupBox* groupBox )
{
	QGridLayout* gridLayout = new QGridLayout( groupBox->layout() );
	gridLayout->setAlignment( Qt::AlignTop );
	return gridLayout;
}
