/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "adjusttimesdialog.h"
#include <timeedit.h>

#include <qlabel.h>
#include <qgroupbox.h>
#include <qlayout.h>

#include <klocale.h>

using namespace SubtitleComposer;


AdjustTimesDialog::AdjustTimesDialog( QWidget* parent ):
	ActionDialog( i18n( "Adjust Subtitles" ), parent )
{
	QGroupBox* buttonGroup = createGroupBox( i18n( "Settings" ) );

	QLabel* firstLineLabel = new QLabel( i18n( "First spoken line" ), buttonGroup );
	firstLineLabel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );

	m_firstLineTimeEdit = new TimeEdit( buttonGroup );

	QLabel* lastLineLabel = new QLabel( i18n( "Last spoken line" ), buttonGroup );
	lastLineLabel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );

	m_lastLineTimeEdit = new TimeEdit( buttonGroup );

	QGridLayout* buttonGroupLayout = createLayout( buttonGroup );
	buttonGroupLayout->addWidget( firstLineLabel, 0, 0 );
	buttonGroupLayout->addWidget( m_firstLineTimeEdit, 0, 1 );
	buttonGroupLayout->addWidget( lastLineLabel, 1, 0 );
	buttonGroupLayout->addWidget( m_lastLineTimeEdit, 1, 1 );
}

Time AdjustTimesDialog::firstLineTime()
{
	return Time( m_firstLineTimeEdit->value() );
}

void AdjustTimesDialog::setFirstLineTime( const Time& time )
{
	m_firstLineTimeEdit->setValue( time.toMillis() );
}

Time AdjustTimesDialog::lastLineTime()
{
	return Time( m_lastLineTimeEdit->value() );
}

void AdjustTimesDialog::setLastLineTime( const Time& time )
{
	m_lastLineTimeEdit->setValue( time.toMillis() );
}
