/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "autodurationsdialog.h"

#include <qlabel.h>
#include <qgroupbox.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qradiobutton.h>

#include <klocale.h>
#include <knuminput.h>


using namespace SubtitleComposer;

AutoDurationsDialog::AutoDurationsDialog( unsigned charMillis, unsigned wordMillis, unsigned lineMillis, QWidget* parent ):
	TargetableTextsActionDialog( i18n( "Automatic Durations" ), parent )
{
	QGroupBox* settingsGroupBox = createGroupBox( i18n( "Settings" ) );

	QLabel* charMillisLabel = new QLabel( i18n( "Milliseconds per character" ), settingsGroupBox );
	charMillisLabel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );

	m_charMillisSpinBox = new KIntSpinBox( settingsGroupBox );
	m_charMillisSpinBox->setMinValue( 0 );
	m_charMillisSpinBox->setMaxValue( 1000 );
	m_charMillisSpinBox->setValue( charMillis );

	QLabel* wordMillisLabel = new QLabel( i18n( "Milliseconds per word" ), settingsGroupBox );
	wordMillisLabel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );

	m_wordMillisSpinBox = new KIntSpinBox( settingsGroupBox );
	m_wordMillisSpinBox->setMinValue( 0 );
	m_wordMillisSpinBox->setMaxValue( 1000 );
	m_wordMillisSpinBox->setValue( wordMillis );

	QLabel* lineMillisLabel = new QLabel( i18n( "Milliseconds per line" ), settingsGroupBox );
	lineMillisLabel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );

	m_lineMillisSpinBox = new KIntSpinBox( settingsGroupBox );
	m_lineMillisSpinBox->setMinValue( 0 );
	m_lineMillisSpinBox->setMaxValue( 1000 );
	m_lineMillisSpinBox->setValue( lineMillis );

	m_preventOverlapCheckBox = new QCheckBox( settingsGroupBox );
	m_preventOverlapCheckBox->setText( i18n( "Prevent overlapping" ) );
	m_preventOverlapCheckBox->setChecked( true );

	QGridLayout* settingsLayout = createLayout( settingsGroupBox );
	settingsLayout->addWidget( charMillisLabel, 0, 0 );
	settingsLayout->addWidget( m_charMillisSpinBox, 0, 1 );
	settingsLayout->addWidget( wordMillisLabel, 1, 0 );
	settingsLayout->addWidget( m_wordMillisSpinBox, 1, 1 );
	settingsLayout->addWidget( lineMillisLabel, 2, 0 );
	settingsLayout->addWidget( m_lineMillisSpinBox, 2, 1 );
	settingsLayout->addMultiCellWidget( m_preventOverlapCheckBox, 3, 3, 0, 1 );

	createTextsTargetGroupBox();

	m_textsTargetGroupBox->setTitle( i18n( "Duration calculation" ) );
	m_textsTargetRadioButtons[SubtitleLine::Primary]->setText( i18n( "Use primary text" ) );
	m_textsTargetRadioButtons[SubtitleLine::Secondary]->setText( i18n( "Use translation text" ) );
	m_textsTargetRadioButtons[SubtitleLine::Both]->setText( i18n( "Calculate both and use maximum" ) );

	createLinesTargetGroupBox();
}

unsigned AutoDurationsDialog::charMillis()
{
	return m_charMillisSpinBox->value();
}

unsigned AutoDurationsDialog::wordMillis()
{
	return m_wordMillisSpinBox->value();
}

unsigned AutoDurationsDialog::lineMillis()
{
	return m_lineMillisSpinBox->value();
}

bool AutoDurationsDialog::preventOverlap()
{
	return m_preventOverlapCheckBox->isChecked();
}
