/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "joinsubtitlesdialog.h"
#include <player.h>
#include <commondefs.h>
#include <timeedit.h>

#include <qlayout.h>
#include <qgroupbox.h>
#include <qtooltip.h>
#include <qcheckbox.h>
#include <qradiobutton.h>

#include <klocale.h>
#include <kpushbutton.h>


using namespace SubtitleComposer;

JoinSubtitlesDialog::JoinSubtitlesDialog( const QString& defaultEncoding, QWidget* parent ):
	SelectableSubtitleDialog( defaultEncoding, i18n( "Join Subtitles" ), parent ),
	m_translationMode( false )
{
	createSubtitleGroupBox();

	m_shiftSubtitleCheckBox = new QCheckBox( m_subtitleGroupBox );
	m_shiftSubtitleCheckBox->setText( i18n( "Shift subtitle forwards before append" ) );
	m_shiftSubtitleCheckBox->setChecked( true );

	m_shiftTimeFromVideoButton = new KPushButton( m_subtitleGroupBox );
	m_shiftTimeFromVideoButton->setIconSet( GUI::customIconSet( "time-from-video", KIcon::Toolbar, 16 ) );
	QToolTip::add( m_shiftTimeFromVideoButton, i18n( "Set from video length" ) );
	connect( m_shiftTimeFromVideoButton, SIGNAL( clicked() ), SLOT( setShiftTimeFromVideo() ) );

	m_shiftTimeEdit = new TimeEdit( m_subtitleGroupBox );
	connect( m_shiftSubtitleCheckBox, SIGNAL( toggled(bool) ), m_shiftTimeFromVideoButton, SLOT( setEnabled(bool) ) );
	connect( m_shiftSubtitleCheckBox, SIGNAL( toggled(bool) ), m_shiftTimeEdit, SLOT( setEnabled(bool) ) );


	m_textsTargetGroupBox = createGroupBox( i18n( "Target subtitle" ) );

	m_primaryTextTargetRadioButton = new QRadioButton( m_textsTargetGroupBox );
	m_primaryTextTargetRadioButton->setText( i18n( "Append to primary subtitle" ) );
	m_primaryTextTargetRadioButton->setChecked( true );

	m_secondaryTextTargetRadioButton = new QRadioButton( m_textsTargetGroupBox );
	m_secondaryTextTargetRadioButton->setText( i18n( "Append to translation subtitle" ) );


	QHBoxLayout* shiftTimeLayout = new QHBoxLayout();
	shiftTimeLayout->addStretch();
	shiftTimeLayout->addWidget( m_shiftTimeFromVideoButton );
	shiftTimeLayout->addWidget( m_shiftTimeEdit );

	m_subtitleLayout->addMultiCellWidget( m_shiftSubtitleCheckBox, 2, 2, 0, 2 );
	m_subtitleLayout->addLayout( shiftTimeLayout, 2, 3 );

	QGridLayout* textsTargetLayout = createLayout( m_textsTargetGroupBox );
	textsTargetLayout->addWidget( m_primaryTextTargetRadioButton, 0, 0 );
	textsTargetLayout->addWidget( m_secondaryTextTargetRadioButton, 1, 0 );


	m_textsTargetGroupBox->hide();
}

void JoinSubtitlesDialog::setShiftTimeFromVideo()
{
	m_shiftTimeEdit->setValue( (int)(Player::instance()->length()*1000 + 0.5) );
}

Time JoinSubtitlesDialog::shiftTime()
{
	return m_shiftSubtitleCheckBox->isChecked() ? m_shiftTimeEdit->value() : 0;
}

void JoinSubtitlesDialog::show()
{
	m_shiftTimeFromVideoButton->setEnabled( Player::instance()->state() > Player::Opening );
	if ( m_shiftTimeFromVideoButton->isEnabled() )
		setShiftTimeFromVideo();

	SelectableSubtitleDialog::show();
}


bool JoinSubtitlesDialog::translationMode()
{
	return m_translationMode;
}

void JoinSubtitlesDialog::setTranslationMode( bool enabled )
{
	if ( m_translationMode != enabled )
	{
		m_translationMode = enabled;

		if ( m_translationMode )
			m_textsTargetGroupBox->show();
		else
		{
			m_primaryTextTargetRadioButton->setChecked( true );
			m_textsTargetGroupBox->hide();
		}

		if ( m_textsTargetGroupBox->layout() )
			m_textsTargetGroupBox->layout()->invalidate();

		QWidget::adjustSize();
		QWidget::resize( width(), 10 );
	}
}

JoinSubtitlesDialog::Target JoinSubtitlesDialog::textTarget()
{
	if ( m_primaryTextTargetRadioButton->isChecked() )
		return Primary;
	else // if ( m_secondaryTextTargetRadioButton->isChecked() )
		return Secondary;
}
