/***************************************************************************
*   Copyright (C) 2007-2008 by Sergio Pistone                             *
*   sergio_pistone@yahoo.com.ar                                           *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "opensubtitledialog.h"
#include <formatsmanager.h>

#include <kdebug.h>
#include <klocale.h>

using namespace SubtitleComposer;

OpenSubtitleDialog::OpenSubtitleDialog( bool primary, const QString& startDir, const QString& encoding, QWidget* parent ):
	KEncodingFileDialog( startDir, encoding, buildInputFormatsFilter(), primary ? i18n( "Open Subtitle" ) : i18n( "Open Translation Subtitle" ), KFileDialog::Opening, parent, 0, true )
{
}

QString OpenSubtitleDialog::buildInputFormatsFilter()
{
	static QString filter;

	if ( filter.isEmpty() )
	{
		QString allSupported;
		QStringList formats = FormatsManager::inputFormatNames();
		for ( QStringList::ConstIterator it = formats.begin(), end = formats.end(); it != end; ++it )
		{
			const InputFormat* format = FormatsManager::inputFormat( *it );
			const QStringList& formatExtensions = format->extensions();
			QString extensions;
			for ( QStringList::ConstIterator extIt = formatExtensions.begin(), extEnd = formatExtensions.end(); extIt != extEnd; ++extIt )
				extensions += "*." + *extIt + " *." + (*extIt).upper();
			allSupported += " " + extensions;
			filter += "\n" + extensions + "|" + format->name();
		}

		filter = allSupported + "|" + i18n( "All Supported" ) + filter;
		filter = filter.stripWhiteSpace();
	}

	return filter;
}
