/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "removelinesdialog.h"

#include <qlayout.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qradiobutton.h>

#include <klocale.h>

using namespace SubtitleComposer;

RemoveLinesDialog::RemoveLinesDialog( bool insertBefore, QWidget* parent ):
	ActionDialog( insertBefore ? i18n( "Remove Line Before Current" ):i18n( "Remove Line After Current" ), parent ),
	m_translationMode( false )
{
	m_textsTargetGroupBox = createGroupBox( i18n( "Target subtitle" ) );

	for ( int index = 0; index < SubtitleLine::OpModeUNKNOWN; ++index )
		m_textsTargetRadioButtons[index] = new QRadioButton( m_textsTargetGroupBox );

	m_textsTargetRadioButtons[SubtitleLine::Primary]->setText( i18n( "Remove only in primary" ) );
	m_textsTargetRadioButtons[SubtitleLine::Secondary]->setText( i18n( "Remove only in translation" ) );
	m_textsTargetRadioButtons[SubtitleLine::Both]->setText( i18n( "Remove in both" ) );

	m_textsTargetRadioButtons[SubtitleLine::Both]->setChecked( true );

	m_textsTargetLayout = createLayout( m_textsTargetGroupBox );
	for ( int index = 0; index < SubtitleLine::OpModeUNKNOWN; ++index )
		m_textsTargetLayout->addWidget( m_textsTargetRadioButtons[index], index, 0 );

	if ( ! m_translationMode )
		m_textsTargetGroupBox->hide();
}

int RemoveLinesDialog::exec()
{
	return m_translationMode ? ActionDialog::exec() : QDialog::Accepted;
}

bool RemoveLinesDialog::translationMode()
{
	return m_translationMode;
}

void RemoveLinesDialog::setTranslationMode( bool enabled )
{
	if ( m_translationMode != enabled )
	{
		m_translationMode = enabled;

		if ( m_translationMode )
			m_textsTargetGroupBox->show();
		else
		{
			m_textsTargetRadioButtons[SubtitleLine::Both]->setChecked( true );
			m_textsTargetGroupBox->hide();
		}

		if ( m_textsTargetGroupBox->layout() )
			m_textsTargetGroupBox->layout()->invalidate();

		QWidget::adjustSize();
		QWidget::resize( width(), 10 );
	}
}

SubtitleLine::OpMode RemoveLinesDialog::textTarget()
{
	if ( m_textsTargetRadioButtons[SubtitleLine::Primary]->isChecked() )
		return SubtitleLine::Primary;
	else if ( m_textsTargetRadioButtons[SubtitleLine::Secondary]->isChecked() )
		return SubtitleLine::Secondary;
	else // if ( m_textsTargetRadioButtons[SubtitleLine::Both]->isChecked() )
		return SubtitleLine::Both;
}
