/***************************************************************************
*   Copyright (C) 2007-2008 by Sergio Pistone                             *
*   sergio_pistone@yahoo.com.ar                                           *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "savesubtitledialog.h"
#include <formatsmanager.h>

#include <klocale.h>
#include <kfilefiltercombo.h>
#include <kcombobox.h>
#include <ktoolbar.h>

using namespace SubtitleComposer;

SaveSubtitleDialog::SaveSubtitleDialog( bool primary, const QString& startDir, const QString& encoding, int newLine, const QString& format, QWidget* parent ):
	KEncodingFileDialog( startDir, encoding, QString::null, primary ? i18n( "Save Subtitle" ) : i18n( "Save Translation Subtitle" ), KFileDialog::Saving, parent, 0, true )
{
	filterWidget->setFilter( buildOutputFormatsFilter() );

	QStringList formatNames = FormatsManager::outputFormatNames();
	if ( formatNames.findIndex( format ) != -1 )
		filterWidget->setCurrentItem( format, false );
	else
	{
		filterWidget->setCurrentItem( formatNames.first(), false );
		slotFilterChanged();
	}

	filterWidget->setEditable( false );

	QStringList newLineModes;
	newLineModes << "unix";
	newLineModes << "windows";
	newLineModes << "macintosh";

	KToolBar* toolBar = this->toolBar();
	toolBar->insertSeparator();
	int index = toolBar->insertCombo( newLineModes, -2 /*id*/, false /*writable*/, 0 /*signal*/, 0 /*receiver*/, 0 /*slot*/ );
	m_newLineComboBox = toolBar->getCombo( toolBar->idAt( index ) );

	if ( m_newLineComboBox )
		m_newLineComboBox->setCurrentItem( newLine );
}

QString SaveSubtitleDialog::selectedFormat()
{
	return filterWidget->currentText();
}

SaveSubtitleDialog::NewLine SaveSubtitleDialog::selectedNewLine()
{
	return m_newLineComboBox ? (NewLine)m_newLineComboBox->currentItem() : SaveSubtitleDialog::CurrentOS;
}

QString SaveSubtitleDialog::buildOutputFormatsFilter()
{
	static QString filter;

	if ( filter.isEmpty() )
	{
		QStringList formats = FormatsManager::outputFormatNames();
		for ( QStringList::ConstIterator it = formats.begin(), end = formats.end(); it != end; ++it )
		{
			const OutputFormat* format = FormatsManager::outputFormat( *it );
			const QStringList& formatExtensions = format->extensions();
			QString extensions;
			for ( QStringList::ConstIterator extIt = formatExtensions.begin(), extEnd = formatExtensions.end(); extIt != extEnd; ++extIt )
				extensions += "*." + *extIt + " *." + (*extIt).upper();
			filter += "\n" + extensions + "|" + format->name();
		}

		filter = filter.stripWhiteSpace();
	}

	return filter;
}
