/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "selectablesubtitledialog.h"
#include "opensubtitledialog.h"
#include <application.h>

#include <qlabel.h>
#include <qgroupbox.h>
#include <qtooltip.h>
#include <qtextcodec.h>
#include <qlayout.h>

#include <kapplication.h>
#include <kiconloader.h>
#include <kpushbutton.h>
#include <kurlcompletion.h>
#include <kcombobox.h>
#include <klineedit.h>


using namespace SubtitleComposer;


SelectableSubtitleDialog::SelectableSubtitleDialog( const QString& defaultEncoding, const QString& title, QWidget* parent ):
	ActionDialog( title, parent ),
	m_defaultEncoding( defaultEncoding )
{
}

QGroupBox* SelectableSubtitleDialog::createSubtitleGroupBox( const QString& title, bool addToLayout )
{
	m_subtitleGroupBox = createGroupBox( title, addToLayout );

	QLabel* subtitlePathLabel = new QLabel( m_subtitleGroupBox );
	subtitlePathLabel->setText( i18n( "Path" ) );

	m_subtitlePathLineEdit = new KLineEdit( m_subtitleGroupBox );
	m_subtitlePathLineEdit->setCompletionObject( new KURLCompletion() );

	KPushButton* subtitleButton = new KPushButton( m_subtitleGroupBox );
	subtitleButton->setIconSet( kapp->iconLoader()->loadIconSet( "fileopen", KIcon::Small ) );
	QToolTip::add( subtitleButton, i18n( "Select subtitle" ) );


	connect( subtitleButton, SIGNAL( clicked() ), SLOT( selectSubtitle() ) );

	QLabel* subtitleEncodingLabel = new QLabel( m_subtitleGroupBox );
	subtitleEncodingLabel->setText( i18n( "Encoding" ) );

	m_subtitleEncodingComboBox = new KComboBox( m_subtitleGroupBox );
	m_subtitleEncodingComboBox->insertStringList( app()->availableEncodingNames() );
	m_subtitleEncodingComboBox->setCurrentItem( m_defaultEncoding );


	QHBoxLayout* subtitlePathLayout = new QHBoxLayout();
	subtitlePathLayout->addWidget( m_subtitlePathLineEdit );
	subtitlePathLayout->addWidget( subtitleButton );

	QHBoxLayout* subtitleEncodingLayout = new QHBoxLayout();
	subtitleEncodingLayout->addStretch();
	subtitleEncodingLayout->addWidget( m_subtitleEncodingComboBox );

	m_subtitleLayout = createLayout( m_subtitleGroupBox );
	m_subtitleLayout->addWidget( subtitlePathLabel, 0, 0 );
	m_subtitleLayout->addMultiCellLayout( subtitlePathLayout, 0, 0, 1, 3 );
	m_subtitleLayout->addWidget( subtitleEncodingLabel, 1, 0 );
	m_subtitleLayout->addMultiCellLayout( subtitleEncodingLayout, 1, 1, 2, 3 );

	return m_subtitleGroupBox;
}

void SelectableSubtitleDialog::selectSubtitle()
{
	OpenSubtitleDialog openDlg(
		true,
		subtitlePath().isEmpty() ?
			app()->lastSubtitleDirectory() :
			subtitlePath(),
		subtitleEncoding()
	);

	if ( openDlg.exec() == QDialog::Accepted )
	{
		m_subtitlePathLineEdit->setText( openDlg.selectedFile() );
		m_subtitleEncodingComboBox->setCurrentItem( openDlg.selectedEncoding().upper() );
	}
}

QString SelectableSubtitleDialog::subtitlePath()
{
	return m_subtitlePathLineEdit->text();
}

QString SelectableSubtitleDialog::subtitleEncoding()
{
	return m_subtitleEncodingComboBox->currentText();
}
