/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "targetablelinesactiondialog.h"

#include <qgroupbox.h>
#include <qradiobutton.h>
#include <qlayout.h>

using namespace SubtitleComposer;

// for derived classes
TargetableLinesActionDialog::TargetableLinesActionDialog( const QString& title, QWidget* parent ):
	ActionDialog( title, parent ),
	m_linesTargetGroupBox( 0 ),
	m_linesTargetLayout( 0 ),
	m_finalClass( false ),
	m_selectedLinesOnlyMode( false ),
	m_selectedLinesOldValue( false )
{
	for ( int index = 0; index < None; ++index )
		m_linesTargetRadioButtons[index] = 0;
}

int TargetableLinesActionDialog::exec()
{
	return ActionDialog::exec();
}

QGroupBox* TargetableLinesActionDialog::createLinesTargetGroupBox( const QString& title, bool addToLayout )
{
	m_linesTargetGroupBox = createGroupBox( title, addToLayout );

	for ( int index = 0; index < None; ++index )
		m_linesTargetRadioButtons[index] = new QRadioButton( m_linesTargetGroupBox );

	m_linesTargetRadioButtons[AllLines]->setText( i18n( "All lines" ) );
	m_linesTargetRadioButtons[Selection]->setText( i18n( "Selected lines" ) );
	m_linesTargetRadioButtons[FromSelected]->setText( i18n( "All lines from first selected" ) );
	m_linesTargetRadioButtons[UpToSelected]->setText( i18n( "All lines up to last selected" ) );

	if ( m_selectedLinesOnlyMode )
	{
		m_linesTargetRadioButtons[Selection]->setChecked( true );
		m_linesTargetGroupBox->hide();
	}
	else
		m_linesTargetRadioButtons[AllLines]->setChecked( true );

	m_linesTargetLayout = createLayout( m_linesTargetGroupBox );
	for ( int index = 0; index < None; ++index )
		m_linesTargetLayout->addWidget( m_linesTargetRadioButtons[index], index, 0 );

	return m_linesTargetGroupBox;
}

bool TargetableLinesActionDialog::isLinesTargetEnabled( Target target )
{
	return m_linesTargetRadioButtons[target] && m_linesTargetRadioButtons[target]->isEnabled();
}

void TargetableLinesActionDialog::setLinesTargetEnabled( Target target, bool enabled )
{
	if ( m_linesTargetRadioButtons[target] )
		m_linesTargetRadioButtons[target]->setEnabled( enabled );
}

bool TargetableLinesActionDialog::selectionTargetOnlyMode()
{
	return m_selectedLinesOnlyMode;
}

void TargetableLinesActionDialog::setSelectionTargetOnlyMode( bool value )
{
	if ( m_selectedLinesOnlyMode != value )
	{
		m_selectedLinesOnlyMode = value;

		if ( ! m_linesTargetGroupBox )
			return;

		if ( m_selectedLinesOnlyMode )
		{
			// save the previous value so we can restore it latter when SelectedLinesOnlyMode is disabled
			m_selectedLinesOldValue = m_linesTargetRadioButtons[Selection]->isChecked();

			m_linesTargetRadioButtons[Selection]->setChecked( true );
			m_linesTargetGroupBox->hide();
		}
		else
		{
			if ( ! m_selectedLinesOldValue )
				m_linesTargetRadioButtons[AllLines]->setChecked( true );

			m_linesTargetGroupBox->show();
		}

		if ( m_linesTargetGroupBox->layout() )
			m_linesTargetGroupBox->layout()->invalidate();

		QWidget::adjustSize();
		QWidget::resize( width(), 10 );
	}
}

TargetableLinesActionDialog::Target TargetableLinesActionDialog::linesTarget()
{
	if ( ! m_linesTargetRadioButtons[Selection] ) // m_linesTargetGroupBox is not created
		return None;

	if ( m_linesTargetRadioButtons[AllLines]->isChecked() )
		return AllLines;
	else if ( m_linesTargetRadioButtons[Selection]->isChecked() )
		return Selection;
	else if ( m_linesTargetRadioButtons[FromSelected]->isChecked() )
		return FromSelected;
	else // if ( m_linesTargetRadioButtons[UpToSelected]->isChecked() )
		return UpToSelected;
}



FinalTargetableLinesActionDialog::FinalTargetableLinesActionDialog( const QString& title, QWidget* parent ):
	TargetableLinesActionDialog( title, parent )
{
	createLinesTargetGroupBox();
}

FinalTargetableLinesActionDialog::FinalTargetableLinesActionDialog( const QString& title, const QString& desc, QWidget* parent ):
	TargetableLinesActionDialog( title, parent )
{
	createLinesTargetGroupBox( desc );
}

int FinalTargetableLinesActionDialog::exec()
{
	return selectionTargetOnlyMode() ? QDialog::Accepted : TargetableLinesActionDialog::exec();
}
