/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "targetabletextsactiondialog.h"

#include <qgroupbox.h>
#include <qradiobutton.h>
#include <qlayout.h>

using namespace SubtitleComposer;

// for derived classes
TargetableTextsActionDialog::TargetableTextsActionDialog( const QString& title, QWidget* parent ):
	TargetableLinesActionDialog( title, parent ),
	m_textsTargetGroupBox( 0 ),
	m_textsTargetLayout( 0 ),
	m_finalClass( false ),
	m_translationMode( false )
{
	for ( int index = 0; index < SubtitleLine::OpModeUNKNOWN; ++index )
		m_textsTargetRadioButtons[index] = 0;
}

int TargetableTextsActionDialog::exec()
{
	return TargetableLinesActionDialog::exec();
}

QGroupBox* TargetableTextsActionDialog::createTextsTargetGroupBox( const QString& title, bool addToLayout )
{
	m_textsTargetGroupBox = createGroupBox( title, addToLayout );

	for ( int index = 0; index < SubtitleLine::OpModeUNKNOWN; ++index )
		m_textsTargetRadioButtons[index] = new QRadioButton( m_textsTargetGroupBox );

	m_textsTargetRadioButtons[SubtitleLine::Primary]->setText( i18n( "Primary" ) );
	m_textsTargetRadioButtons[SubtitleLine::Secondary]->setText( i18n( "Translation" ) );
	m_textsTargetRadioButtons[SubtitleLine::Both]->setText( i18n( "Both" ) );

	if ( m_translationMode )
		m_textsTargetRadioButtons[SubtitleLine::Both]->setChecked( true );
	else
	{
		m_textsTargetRadioButtons[SubtitleLine::Primary]->setChecked( true );
		m_textsTargetGroupBox->hide();
	}

	m_textsTargetLayout = createLayout( m_textsTargetGroupBox );
	for ( int index = 0; index < SubtitleLine::OpModeUNKNOWN; ++index )
		m_textsTargetLayout->addWidget( m_textsTargetRadioButtons[index], index, 0 );

	return m_textsTargetGroupBox;
}

bool TargetableTextsActionDialog::isTextTargetEnabled( SubtitleLine::OpMode target )
{
	return m_textsTargetRadioButtons[target] && m_textsTargetRadioButtons[target]->isEnabled();
}

void TargetableTextsActionDialog::setTextTargetEnabled( SubtitleLine::OpMode target, bool enabled )
{
	if ( m_textsTargetRadioButtons[target] )
		m_textsTargetRadioButtons[target]->setEnabled( enabled );
}

bool TargetableTextsActionDialog::translationMode()
{
	return m_translationMode;
}

void TargetableTextsActionDialog::setTranslationMode( bool enabled )
{
	if ( m_translationMode != enabled )
	{
		m_translationMode = enabled;

		if ( ! m_textsTargetGroupBox )
			return;

		if ( m_translationMode )
		{
			m_textsTargetRadioButtons[SubtitleLine::Both]->setChecked( true );
			m_textsTargetGroupBox->show();
		}
		else
		{
			m_textsTargetRadioButtons[SubtitleLine::Primary]->setChecked( true );
			m_textsTargetGroupBox->hide();
		}

		if ( m_textsTargetGroupBox->layout() )
			m_textsTargetGroupBox->layout()->invalidate();

		QWidget::adjustSize();
		QWidget::resize( width(), 10 );
	}
}

SubtitleLine::OpMode TargetableTextsActionDialog::textTarget()
{
	if ( ! m_textsTargetRadioButtons[SubtitleLine::Primary] ) // m_textsTargetGroupBox is not created
		return SubtitleLine::OpModeUNKNOWN;

	if ( m_textsTargetRadioButtons[SubtitleLine::Primary]->isChecked() )
		return SubtitleLine::Primary;
	else if ( m_textsTargetRadioButtons[SubtitleLine::Secondary]->isChecked() )
		return SubtitleLine::Secondary;
	else // if ( m_textsTargetRadioButtons[SubtitleLine::Both]->isChecked() )
		return SubtitleLine::Both;
}



FinalTargetableTextsActionDialog::FinalTargetableTextsActionDialog( const QString& title, QWidget* parent ):
	TargetableTextsActionDialog( title, parent )
{
	createTextsTargetGroupBox();
	createLinesTargetGroupBox();
}

FinalTargetableTextsActionDialog::FinalTargetableTextsActionDialog( const QString& title, const QString& lDesc, const QString& tDesc, QWidget* parent ):
	TargetableTextsActionDialog( title, parent )
{
	createTextsTargetGroupBox( tDesc );
	createLinesTargetGroupBox( lDesc );
}

int FinalTargetableTextsActionDialog::exec()
{
	return selectionTargetOnlyMode() && ! translationMode() ? QDialog::Accepted : TargetableLinesActionDialog::exec();
}
