#ifndef ERRORSWIDGET_H
#define ERRORSWIDGET_H

/***************************************************************************
*   Copyright (C) 2007-2008 by Sergio Pistone                             *
*   sergio_pistone@yahoo.com.ar                                           *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include "errorswidgetitem.h"
#include <ranges.h>
#include <subtitle.h>

#include <klistview.h>

namespace SubtitleComposer
{
	class LinesWidget;
	class ErrorsWidgetItem;

	class ErrorsWidget : public KListView
	{
		Q_OBJECT

		friend class ErrorsWidgetItem;

		public:

			ErrorsWidget( QWidget* parent );
			~ErrorsWidget();

			ErrorsWidgetItem* item( unsigned index );
			SubtitleLine* itemLine( QListViewItem* item );

			SubtitleLine* currentLine();

			int lineSelectedErrorFlags( unsigned index );

			RangeList selectionRanges() const;

			unsigned linesCount();
			unsigned errorsCount();
			unsigned marksCount();

		public slots:

			void setSubtitle( Subtitle* subtitle=0 );

			void setCurrentLine( SubtitleLine* line );

		signals:

			void currentLineChanged( SubtitleLine* line );
			void lineDoubleClicked( SubtitleLine* line );

			void linesCountChanged( unsigned linesCount );
			void errorsCountChanged( unsigned errorsCount );
			void marksCountChanged( unsigned marksCount );

		protected:

			void setCurrentItem( QListViewItem* currentItem, bool clearSelection );

			ErrorsWidgetItem* createItem( SubtitleLine* line, ErrorsWidgetItem* after, unsigned number=0 );

			virtual void contextMenuEvent( QContextMenuEvent* e );
			virtual void contentsMouseDoubleClickEvent( QMouseEvent* e );
			virtual void keyPressEvent( QKeyEvent* e );

			void updateLinesCount( bool increment );
			void updateErrorsCount( bool increment );
			void updateMarksCount( bool increment );

		protected slots:

			void onLinesInserted( unsigned firstIndex, unsigned lastIndex );
			void onLinesRemoved( unsigned firstIndex, unsigned lastIndex );

			void onCurrentChanged( QListViewItem* currentItem );

		protected:

			Subtitle* m_subtitle;

			unsigned m_linesCount;
			unsigned m_errorsCount;
			unsigned m_marksCount;
	};
}

#endif
