/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "application.h"
#include "errorswidget.h"
#include "errorswidgetitem.h"
#include "configdefs/errorsconfigdefs.h"
#include "subtitleline.h"

#include <kapplication.h>
#include <kiconloader.h>
#include <klocale.h>

using namespace SubtitleComposer;

/// ErrorsWidgetItem

ErrorsWidgetItem::ErrorsWidgetItem( SubtitleLine* line, ErrorsWidget* parent, ErrorsWidgetItem* after ):
	QObject( 0 ),
	KListViewItem( parent, after ),
	m_line( line )
{
	for ( int id = 0; id <= SubtitleLine::ErrorSIZE; ++id )
		m_items[id] = 0;

	int errorFlags = m_line->errorFlags();
	bool marked = m_line->marked();

	for ( int id = 0; id < SubtitleLine::ErrorSIZE; ++id )
		setupItem( (SubtitleLine::ErrorID)id, errorFlags & SubtitleLine::errorFlag( (SubtitleLine::ErrorID)id ) );
	setupItem( SubtitleLine::ErrorSIZE, marked );

	setVisible( errorFlags || marked );
	if ( isVisible() )
	{
		errorsWidget()->updateLinesCount( true );
		if ( errorFlags )
			errorsWidget()->updateErrorsCount( true );
		if ( marked )
			errorsWidget()->updateMarksCount( true );
	}

	connect( line, SIGNAL( errorFlagsChanged(int) ), this, SLOT( onErrorFlagsChanged(int) ) );
	connect( line, SIGNAL( markChanged(bool) ), this, SLOT( onMarkChanged(bool) ) );

	connect( line, SIGNAL( primaryTextChanged(const QString&) ), this, SLOT( onPrimaryTextChanged(const QString&) ) );
	connect( line, SIGNAL( secondaryTextChanged(const QString&) ), this, SLOT( onSecondaryTextChanged(const QString&) ) );
	connect( line, SIGNAL( showTimeChanged(const Time&) ), this, SLOT( onTimesChanged(const Time&) ) );
	connect( line, SIGNAL( hideTimeChanged(const Time&) ), this, SLOT( onTimesChanged(const Time&) ) );
}

ErrorsWidgetItem::~ErrorsWidgetItem()
{
}

ErrorsWidget* ErrorsWidgetItem::errorsWidget()
{
	return static_cast<ErrorsWidget*>( listView() );
}

SubtitleLine* ErrorsWidgetItem::line()
{
	return m_line;
}

void ErrorsWidgetItem::setNumber( unsigned number )
{
	setText( 0, i18n( "Line %1" ).arg( number ) );
}

ErrorsWidgetItem* ErrorsWidgetItem::nextSiblingCasted()
{
	return static_cast<ErrorsWidgetItem*>( nextSibling() );
}

void ErrorsWidgetItem::onPrimaryTextChanged( const QString& )
{
	if ( m_items[SubtitleLine::EmptyPrimaryTextID] )
		m_items[SubtitleLine::EmptyPrimaryTextID]->update();

	if ( m_items[SubtitleLine::MinDurationPerPrimaryCharID] )
		m_items[SubtitleLine::MinDurationPerPrimaryCharID]->update();

	if ( m_items[SubtitleLine::MaxDurationPerPrimaryCharID] )
		m_items[SubtitleLine::MaxDurationPerPrimaryCharID]->update();

	if ( m_items[SubtitleLine::MaxPrimaryCharsID] )
		m_items[SubtitleLine::MaxPrimaryCharsID]->update();

	if ( m_items[SubtitleLine::MaxPrimaryLinesID] )
		m_items[SubtitleLine::MaxPrimaryLinesID]->update();

	if ( m_items[SubtitleLine::PrimaryUnneededSpacesID] )
		m_items[SubtitleLine::PrimaryUnneededSpacesID]->update();

	if ( m_items[SubtitleLine::PrimaryCapitalAfterEllipsisID] )
		m_items[SubtitleLine::PrimaryCapitalAfterEllipsisID]->update();

	if ( m_items[SubtitleLine::PrimaryUnneededDashID] )
		m_items[SubtitleLine::PrimaryUnneededDashID]->update();

	updateLineErrors( m_line, m_line->errorFlags() & SubtitleLine::PrimaryErrors );
}

void ErrorsWidgetItem::onSecondaryTextChanged( const QString& )
{
	if ( m_items[SubtitleLine::EmptySecondaryTextID] )
		m_items[SubtitleLine::EmptySecondaryTextID]->update();

	if ( m_items[SubtitleLine::MinDurationPerSecondaryCharID] )
		m_items[SubtitleLine::MinDurationPerSecondaryCharID]->update();

	if ( m_items[SubtitleLine::MaxDurationPerSecondaryCharID] )
		m_items[SubtitleLine::MaxDurationPerSecondaryCharID]->update();

	if ( m_items[SubtitleLine::MaxSecondaryCharsID] )
		m_items[SubtitleLine::MaxSecondaryCharsID]->update();

	if ( m_items[SubtitleLine::MaxSecondaryLinesID] )
		m_items[SubtitleLine::MaxSecondaryLinesID]->update();

	if ( m_items[SubtitleLine::SecondaryUnneededSpacesID] )
		m_items[SubtitleLine::SecondaryUnneededSpacesID]->update();

	if ( m_items[SubtitleLine::SecondaryCapitalAfterEllipsisID] )
		m_items[SubtitleLine::SecondaryCapitalAfterEllipsisID]->update();

	if ( m_items[SubtitleLine::SecondaryUnneededDashID] )
		m_items[SubtitleLine::SecondaryUnneededDashID]->update();

	updateLineErrors( m_line, m_line->errorFlags() & SubtitleLine::SecondaryErrors );
}

void ErrorsWidgetItem::onTimesChanged( const Time& )
{
	if ( m_items[SubtitleLine::MinDurationID] )
		m_items[SubtitleLine::MinDurationID]->update();

	if ( m_items[SubtitleLine::MaxDurationID] )
		m_items[SubtitleLine::MaxDurationID]->update();

	if ( m_items[SubtitleLine::MinDurationPerPrimaryCharID] )
		m_items[SubtitleLine::MinDurationPerPrimaryCharID]->update();

	if ( m_items[SubtitleLine::MinDurationPerSecondaryCharID] )
		m_items[SubtitleLine::MinDurationPerSecondaryCharID]->update();

	if ( m_items[SubtitleLine::MaxDurationPerPrimaryCharID] )
		m_items[SubtitleLine::MaxDurationPerPrimaryCharID]->update();

	if ( m_items[SubtitleLine::MaxDurationPerSecondaryCharID] )
		m_items[SubtitleLine::MaxDurationPerSecondaryCharID]->update();

	updateLineErrors( m_line, m_line->errorFlags() & SubtitleLine::TimesErrors );
	if ( m_line->prevLine() )
		updateLineErrors( m_line->prevLine(), m_line->prevLine()->errorFlags() & SubtitleLine::OverlapsWithNext );
}

void ErrorsWidgetItem::updateLineErrors( SubtitleLine* line, int errorFlags )
{
	if ( app()->errorsConfig()->optionAsBool( OPT_AUTO_CLEAR_FIXED ) )
	{
		line->check(
			errorFlags,
			app()->errorsConfig()->optionAsInt( OPT_MIN_DURATION ),
			app()->errorsConfig()->optionAsInt( OPT_MAX_DURATION ),
			app()->errorsConfig()->optionAsInt( OPT_MIN_DURATION_PER_CHARACTER ),
			app()->errorsConfig()->optionAsInt( OPT_MAX_DURATION_PER_CHARACTER ),
			app()->errorsConfig()->optionAsInt( OPT_MAX_CHARACTERS ),
			app()->errorsConfig()->optionAsInt( OPT_MAX_LINES )
		);
	}
}



void ErrorsWidgetItem::setupItem( SubtitleLine::ErrorID id, bool active )
{
	if ( m_items[id] )
	{
		if ( ! active )
		{
			delete m_items[id];
			m_items[id] = 0;
		}
	}
	else
	{
		if ( active )
		{
			m_items[id] = new ErrorsWidgetNestedItem( id, this );
			setOpen( true );
		}
	}
}

void ErrorsWidgetItem::onErrorFlagsChanged( int errorFlags )
{
	bool hadErrorFlags = false; // we have to extract information about the previous errorFlags state
	for ( int id = 0; id < SubtitleLine::ErrorSIZE; ++id )
	{
		if ( m_items[id] )
			hadErrorFlags = true;
		setupItem( (SubtitleLine::ErrorID)id, errorFlags & SubtitleLine::errorFlag( (SubtitleLine::ErrorID)id ) );
	}

	bool visible = errorFlags || m_line->marked();
	if ( visible != isVisible() )
	{
		errorsWidget()->updateLinesCount( visible );
		setVisible( visible );
	}

	bool hasErrorFlags = !!errorFlags;
	if ( hadErrorFlags != hasErrorFlags )
		errorsWidget()->updateErrorsCount( hasErrorFlags );

}

void ErrorsWidgetItem::onMarkChanged( bool marked )
{
	bool wasMarked = m_items[SubtitleLine::ErrorSIZE]; // we have to extract information about the previous marked state
	setupItem( SubtitleLine::ErrorSIZE, marked );

	bool visible = m_line->errorFlags() || marked;
	if ( visible != isVisible() )
	{
		errorsWidget()->updateLinesCount( visible );
		setVisible( visible );
	}

	if ( wasMarked != marked )
		errorsWidget()->updateMarksCount( marked );
}



/// ErrorsWidgetNestedItem

ErrorsWidgetNestedItem::ErrorsWidgetNestedItem( SubtitleLine::ErrorID id, ErrorsWidgetItem* parent ):
	KListViewItem( parent ),
	m_id( id )
{
	update();

	setPixmap( 0, id == SubtitleLine::ErrorSIZE ? markPixmap() : errorPixmap() );
}

ErrorsWidgetNestedItem::~ErrorsWidgetNestedItem()
{
}

ErrorsWidgetItem* ErrorsWidgetNestedItem::parent()
{
	return static_cast<ErrorsWidgetItem*>( KListViewItem::parent() );
}

int ErrorsWidgetNestedItem::errorFlags()
{
	return SubtitleLine::errorFlag( m_id );
}

void ErrorsWidgetNestedItem::update()
{
	QString text( SubtitleLine::errorText( m_id, true ) );

	switch ( m_id )
	{
		case SubtitleLine::MaxDurationID:
			text = text.arg( parent()->m_line->durationTime().toMillis() );
			break;
		case SubtitleLine::MinDurationID:
			text = text.arg( parent()->m_line->durationTime().toMillis() );
			break;
		case SubtitleLine::MaxDurationPerPrimaryCharID:
			text = text.arg( parent()->m_line->durationTime().toMillis()/parent()->m_line->primaryCharacters() );
			break;
		case SubtitleLine::MaxDurationPerSecondaryCharID:
			text = text.arg( parent()->m_line->durationTime().toMillis()/parent()->m_line->secondaryCharacters() );
			break;
		case SubtitleLine::MinDurationPerPrimaryCharID:
			text = text.arg( parent()->m_line->durationTime().toMillis()/parent()->m_line->primaryCharacters() );
			break;
		case SubtitleLine::MinDurationPerSecondaryCharID:
			text = text.arg( parent()->m_line->durationTime().toMillis()/parent()->m_line->secondaryCharacters() );
			break;
		case SubtitleLine::MaxPrimaryCharsID:
			text = text.arg( parent()->m_line->primaryCharacters() );
			break;
		case SubtitleLine::MaxSecondaryCharsID:
			text = text.arg( parent()->m_line->secondaryCharacters() );
			break;
		case SubtitleLine::MaxPrimaryLinesID:
			text = text.arg( parent()->m_line->primaryLines() );
			break;
		case SubtitleLine::MaxSecondaryLinesID:
			text = text.arg( parent()->m_line->secondaryLines() );
			break;
		case SubtitleLine::ErrorSIZE:
			text = i18n( "Has user mark." );
			break;
		default:
			break;
	}

	setText( 0, text );
}

const QPixmap& ErrorsWidgetNestedItem::markPixmap()
{
	static QPixmap okPixmap;

	if ( okPixmap.isNull() )
		okPixmap = kapp->iconLoader()->loadIcon( "messagebox_warning", KIcon::Desktop, 16 );

	return okPixmap;
}

const QPixmap& ErrorsWidgetNestedItem::errorPixmap()
{
	static QPixmap errorPixmap;

	if ( errorPixmap.isNull() )
		errorPixmap = kapp->iconLoader()->loadIcon( "messagebox_critical", KIcon::Desktop, 16 );

	return errorPixmap;
}

#include "errorswidgetitem.moc"
