#ifndef ERRORSWIDGETITEM_H
#define ERRORSWIDGETITEM_H

/***************************************************************************
*   Copyright (C) 2007-2008 by Sergio Pistone                             *
*   sergio_pistone@yahoo.com.ar                                           *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <subtitleline.h>

#include <qobject.h>
#include <klistview.h>
#include <appconfiggroup.h>

namespace SubtitleComposer
{
	class ErrorsWidget;
	class ErrorsWidgetNestedItem;

	class ErrorsWidgetItem : public QObject, public KListViewItem
	{
		Q_OBJECT

		friend class ErrorsWidgetNestedItem;

		public:

			ErrorsWidgetItem( SubtitleLine* line, ErrorsWidget* parent, ErrorsWidgetItem* after );
			virtual ~ErrorsWidgetItem();

			ErrorsWidget* errorsWidget();

			SubtitleLine* line();

			void setNumber( unsigned number );

			ErrorsWidgetItem* nextSiblingCasted();

		private:

			void setupItem( SubtitleLine::ErrorID id, bool active );
			static void updateLineErrors( SubtitleLine* line, int errorFlags );

		private slots:

			void onErrorFlagsChanged( int errorFlags );
			void onMarkChanged( bool marked );

			void onPrimaryTextChanged( const QString& text );
			void onSecondaryTextChanged( const QString& text );
			void onTimesChanged( const Time& time );

		private:

			SubtitleLine* m_line;

			ErrorsWidgetNestedItem* m_items[SubtitleLine::ErrorSIZE+1];
	};

	class ErrorsWidgetNestedItem : public KListViewItem
	{
		public:

			friend class ErrorsWidgetItem;

			ErrorsWidgetNestedItem( SubtitleLine::ErrorID id, ErrorsWidgetItem* parent );
			virtual ~ErrorsWidgetNestedItem();

			ErrorsWidgetItem* parent();

			int errorFlags();

		private:

			static SubtitleLine::ErrorFlag errorFlag( SubtitleLine::ErrorID id );

			void update();

			const QPixmap& markPixmap();
			const QPixmap& errorPixmap();

		private:

			SubtitleLine::ErrorID m_id;
	};
}

#endif
