#ifndef LINESWIDGET_H
#define LINESWIDGET_H

/***************************************************************************
*   Copyright (C) 2007-2008 by Sergio Pistone                             *
*   sergio_pistone@yahoo.com.ar                                           *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <appconfig.h>
#include <ranges.h>
#include <subtitle.h>
#include <subtitleline.h>

#include <klistview.h>

namespace SubtitleComposer
{
	class Subtitle;
	class SubtitleLine;
	class LinesWidgetItem;
	class ConfigDialog;
	class ErrorsDialog;

	class LinesWidget : public KListView
	{
		Q_OBJECT

		friend class LinesWidgetItem;

		enum { Number=0, ShowTime, HideTime, Text, Translation };

		public:

			LinesWidget( QWidget* parent );
			~LinesWidget();

			bool showingContextMenu();

			bool currentFollowsLinesChanges();
			void setCurrentFollowsLinesChanges( bool value );

			LinesWidgetItem* item( unsigned index );
			int itemIndex( QListViewItem* item );

			SubtitleLine* currentLine();
			int currentIndex();

			int firstSelectedIndex() const;
			int lastSelectedIndex() const;
			bool selectionHasMultipleRanges() const;
			RangeList selectionRanges() const;
			RangeList targetRanges( int target ) const;

			void loadConfig();
			void saveConfig();

			virtual bool eventFilter( QObject* object, QEvent* event );

		public slots:

			void setSubtitle( Subtitle* subtitle=0 );
			void setTranslationMode( bool enabled );

			void setCurrentLine( SubtitleLine* line, bool clearSelection=true );
			void setPlayingLine( SubtitleLine* line );

			void editCurrentLinePrimaryInPlace();
			void editCurrentLineSecondaryInPlace();

		signals:

			void currentLineChanged( SubtitleLine* line );
			void lineDoubleClicked( SubtitleLine* line );

		private:

			virtual void contextMenuEvent( QContextMenuEvent* e );
			virtual void contentsMouseDoubleClickEvent( QMouseEvent* e );

			void setCurrentItem( QListViewItem* currentItem, bool clearSelection=true, bool force=false );

			LinesWidgetItem* createItem( SubtitleLine* line, LinesWidgetItem* after, unsigned number=0 );

		private slots:

			void onCurrentChanged( QListViewItem* currentItem );

			void onLinesInserted( unsigned firstIndex, unsigned lastIndex );
			void onLinesRemoved( unsigned firstIndex, unsigned lastIndex );

		private:

			Subtitle* m_subtitle;
			bool m_translationMode;

			LinesWidgetItem* m_playingItem;

			bool m_nextItemCurrentOnDelete;
			bool m_currentFollowsLinesChanges;

			bool m_showingContextMenu;
	};

}

#endif
