/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "playerwidget.h"
#include "application.h"
#include "configdefs/playerconfigdefs.h"
#include "useractions/useractionnames.h"
#include <player.h>
#include <playerbackend.h>
#include <commondefs.h>
#include <subtitleiterator.h>

#include <layeredwidget.h>
#include <textoverlaywidget.h>
#include <attachablewidget.h>
#include <pointingslider.h>
#include <timeedit.h>

#include <qevent.h>
#include <qlayout.h>
#include <qcursor.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>

#include <klocale.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <kpopupmenu.h>
#include <kconfig.h>
#include <kurldrag.h>


using namespace SubtitleComposer;

// FIXME WTF is this!??
#define MAGIC_NUMBER -1

#define UNKNOWN_LENGTH_STRING	(" / " + Time().toString() + " ")

PlayerWidget::PlayerWidget( QWidget* parent ):
	QWidget( parent ),
	m_subtitle( 0 ),
	m_translationMode( false ),
	m_showTranslation( false ),
	m_overlayLine( 0 ),
	m_playingLine( 0 ),
	m_fullScreenMode( false ),
	m_player( Player::instance() ),
	m_lengthString( UNKNOWN_LENGTH_STRING ),
	m_updatePositionControls( 1 ),
	m_updateVideoPosition( false ),
	m_updateVolumeControls( true ),
	m_updatePlayerVolume( false ),
	m_showPositionEdit( false )
{
	m_layeredWidget = new LayeredWidget( this );
	m_layeredWidget->setAcceptDrops( true );
	m_layeredWidget->installEventFilter( this );
	m_layeredWidget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );

	m_player->initBackend( m_layeredWidget, app()->playerConfig()->option( OPT_PLAYER_BACKEND ) );

	connect( m_player, SIGNAL(backendInitialized(PlayerBackend*)), this, SLOT( onPlayerBackendInitialized() ) );

	m_textOverlay = new TextOverlayWidget( m_layeredWidget );

	m_seekSlider = new PointingSlider( Qt::Horizontal, this );
	m_seekSlider->setTickmarks( QSlider::NoMarks );
	m_seekSlider->setTracking( false );
	m_seekSlider->setMinValue( 0 );
	m_seekSlider->setMaxValue( 1000 );
	m_seekSlider->setPageStep( 10 );

	m_infoControlsGroupBox = new QGroupBox( this );
// 	m_infoControlsGroupBox->setTitle( i18n( "Information" ) );
	m_infoControlsGroupBox->setAcceptDrops( true );
	m_infoControlsGroupBox->installEventFilter( this );

	QLabel* positionTagLabel = new QLabel( m_infoControlsGroupBox );
	positionTagLabel->setText( i18n( "<b>Position</b>" ) );
	positionTagLabel->installEventFilter( this );

	m_positionLabel = new QLabel( m_infoControlsGroupBox );
	m_positionEdit = new TimeEdit( m_infoControlsGroupBox );

	QLabel* lengthTagLabel = new QLabel( m_infoControlsGroupBox );
	lengthTagLabel->setText( i18n( "<b>Length</b>") );
	lengthTagLabel->installEventFilter( this );
	m_lengthLabel = new QLabel( m_infoControlsGroupBox );

	QLabel* fpsTagLabel = new QLabel( m_infoControlsGroupBox );
	fpsTagLabel->setText( i18n( "<b>FPS</b>") );
	fpsTagLabel->installEventFilter( this );
	m_fpsLabel = new QLabel( m_infoControlsGroupBox );

	m_fpsLabel->setMinimumWidth( m_positionEdit->sizeHint().width() ); // sets the minimum width for the whole group

	m_volumeSlider = new PointingSlider( Qt::Vertical, this );
	m_volumeSlider->setFocusPolicy( QWidget::NoFocus );
	m_volumeSlider->setTickmarks( QSlider::NoMarks );
	m_volumeSlider->setTracking( true );
	m_volumeSlider->setPageStep( 5 );
	m_volumeSlider->setMinValue( 0 );
	m_volumeSlider->setMaxValue( 100 );

	QGridLayout* videoControlsLayout = new QGridLayout();
	videoControlsLayout->setSpacing( 2 );
	videoControlsLayout->addWidget( createToolButton( this, ACT_PLAY_PAUSE, 16 ), 0, 0 );
	videoControlsLayout->addWidget( createToolButton( this, ACT_STOP, 16 ), 0, 1 );
	videoControlsLayout->addWidget( createToolButton( this, ACT_SEEK_BACKWARDS, 16 ), 0, 2 );
	videoControlsLayout->addWidget( createToolButton( this, ACT_SEEK_FORWARDS, 16 ), 0, 3 );
	videoControlsLayout->addItem( new QSpacerItem( 2, 2 ), 0, 4 );
	videoControlsLayout->addWidget( createToolButton( this, ACT_SEEK_TO_PREVIOUS_LINE, 16 ), 0, 5 );
	videoControlsLayout->addWidget( createToolButton( this, ACT_SEEK_TO_NEXT_LINE, 16 ), 0, 6 );
	videoControlsLayout->addItem( new QSpacerItem( 2, 2 ), 0, 7 );
	videoControlsLayout->addWidget( createToolButton( this, ACT_SET_CURRENT_LINE_SHOW_TIME, 16 ), 0, 8 );
	videoControlsLayout->addWidget( createToolButton( this, ACT_SET_CURRENT_LINE_HIDE_TIME, 16 ), 0, 9 );
	videoControlsLayout->addItem( new QSpacerItem( 2, 2 ), 0, 10 );
	videoControlsLayout->addWidget( createToolButton( this, ACT_CURRENT_LINE_FOLLOWS_VIDEO, 16 ), 0, 11 );
	videoControlsLayout->addWidget( m_seekSlider, 0, 12 );

	QGridLayout* audioControlsLayout = new QGridLayout();
	audioControlsLayout->addWidget( createToolButton( this, ACT_TOGGLE_MUTED, 16 ), 0, 0, Qt::AlignHCenter );
	audioControlsLayout->addWidget( m_volumeSlider, 1, 0, Qt::AlignHCenter );

	QGridLayout* infoControlsLayout = new QGridLayout( m_infoControlsGroupBox );
	infoControlsLayout->setSpacing( 5 );
	infoControlsLayout->setMargin( 5 );
	infoControlsLayout->addWidget( fpsTagLabel, 0, 0 );
	infoControlsLayout->addWidget( m_fpsLabel, 1, 0 );
	infoControlsLayout->addItem( new QSpacerItem( 1, 1, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding ), 2, 0 );
	infoControlsLayout->addWidget( lengthTagLabel, 3, 0 );
	infoControlsLayout->addWidget( m_lengthLabel, 4, 0 );
	infoControlsLayout->addItem( new QSpacerItem( 1, 1, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding ), 5, 0 );
	infoControlsLayout->addWidget( positionTagLabel, 6, 0 );
	infoControlsLayout->addWidget( m_positionLabel, 7, 0 );
	infoControlsLayout->addWidget( m_positionEdit, 8, 0 );

	m_mainLayout = new QGridLayout( this );
	m_mainLayout->setSpacing( 5 );
	m_mainLayout->addMultiCellWidget( m_infoControlsGroupBox, 0, 1, 0, 0 );
	m_mainLayout->addWidget( m_layeredWidget, 0, 1 );
	m_mainLayout->addLayout( audioControlsLayout, 0, 2 );
	m_mainLayout->addLayout( videoControlsLayout, 1, 1 );


	m_fullScreenControls = new AttachableWidget();
	m_layeredWidget->setWidgetMode( m_fullScreenControls, LayeredWidget::Ignore );

	m_fsSeekSlider = new PointingSlider( Qt::Horizontal, m_fullScreenControls );
	m_fsSeekSlider->setTickmarks( QSlider::NoMarks );
	m_fsSeekSlider->setTracking( false );
	m_fsSeekSlider->setMinValue( 0 );
	m_fsSeekSlider->setMaxValue( 1000 );
	m_fsSeekSlider->setPageStep( 10 );

	m_fsVolumeSlider = new PointingSlider( Qt::Horizontal, m_fullScreenControls );
	m_fsVolumeSlider->setFocusPolicy( QWidget::NoFocus );
	m_fsVolumeSlider->setTickmarks( QSlider::NoMarks );
	m_fsVolumeSlider->setTracking( true );
	m_fsVolumeSlider->setPageStep( 5 );
	m_fsVolumeSlider->setMinValue( 0 );
	m_fsVolumeSlider->setMaxValue( 100 );

	m_fsPositionLabel = new QLabel( m_fullScreenControls );
	m_fsPositionLabel->setBackgroundColor( Qt::black );
	QPalette palette( m_fsPositionLabel->palette() );
	palette.setColor( QColorGroup::Background, Qt::black );
	palette.setColor( QColorGroup::Foreground, Qt::white );
	m_fsPositionLabel->setPalette( palette );
    m_fsPositionLabel->setFrameShape( QFrame::Panel );
    m_fsPositionLabel->setFrameShadow( QFrame::Sunken );

	QHBoxLayout* fullScreenControlsLayout = new QHBoxLayout( m_fullScreenControls );
	fullScreenControlsLayout->setMargin( 1 );
	fullScreenControlsLayout->setSpacing( 2 );

	fullScreenControlsLayout->addWidget( createToolButton( m_fullScreenControls, ACT_PLAY_PAUSE, 32 ) );
	fullScreenControlsLayout->addWidget( createToolButton( m_fullScreenControls, ACT_STOP, 32 ) );
	fullScreenControlsLayout->addWidget( createToolButton( m_fullScreenControls, ACT_SEEK_BACKWARDS, 32 ) );
	fullScreenControlsLayout->addWidget( createToolButton( m_fullScreenControls, ACT_SEEK_FORWARDS, 32 ) );
	fullScreenControlsLayout->addSpacing( 3 );
	fullScreenControlsLayout->addWidget( createToolButton( m_fullScreenControls, ACT_SEEK_TO_PREVIOUS_LINE, 32 ) );
	fullScreenControlsLayout->addWidget( createToolButton( m_fullScreenControls, ACT_SEEK_TO_NEXT_LINE, 32 ) );
	fullScreenControlsLayout->addSpacing( 3 );
	fullScreenControlsLayout->addWidget( m_fsSeekSlider, 9 );
	fullScreenControlsLayout->addWidget( m_fsPositionLabel );
// 	fullScreenControlsLayout->addSpacing( 1 );
	fullScreenControlsLayout->addWidget( createToolButton( m_fullScreenControls, ACT_TOGGLE_MUTED, 32 ) );
	fullScreenControlsLayout->addWidget( m_fsVolumeSlider, 2 );
	m_fullScreenControls->adjustSize();

	connect( m_volumeSlider, SIGNAL( valueChanged(int) ), this, SLOT(onVolumeSliderValueChanged(int)) );
	connect( m_fsVolumeSlider, SIGNAL( valueChanged(int) ), this, SLOT(onVolumeSliderValueChanged(int)) );

	connect( m_seekSlider, SIGNAL( valueChanged(int) ), this, SLOT(onSeekSliderValueChanged(int)) );
	connect( m_seekSlider, SIGNAL( sliderMoved(int) ), this, SLOT(onSeekSliderMoved(int)) );
	connect( m_seekSlider, SIGNAL( sliderPressed() ), this, SLOT(onSeekSliderPressed()) );
	connect( m_seekSlider, SIGNAL( sliderReleased() ), this, SLOT(onSeekSliderReleased()) );
	connect( m_fsSeekSlider, SIGNAL( valueChanged(int) ), this, SLOT(onSeekSliderValueChanged(int)) );
	connect( m_fsSeekSlider, SIGNAL( sliderMoved(int) ), this, SLOT(onSeekSliderMoved(int)) );
	connect( m_fsSeekSlider, SIGNAL( sliderPressed() ), this, SLOT(onSeekSliderPressed()) );
	connect( m_fsSeekSlider, SIGNAL( sliderReleased() ), this, SLOT(onSeekSliderReleased()) );

	connect( m_positionEdit, SIGNAL( valueChanged(int) ), this, SLOT(onPositionEditValueChanged(int)) );
	connect( m_positionEdit, SIGNAL( valueEntered(int) ), this, SLOT(onPositionEditValueChanged(int)) );

	connect( app()->playerConfig(), SIGNAL(optionChanged(const QString&,const QString&)),
			 this, SLOT(onPlayerOptionChanged(const QString&,const QString&)) );

	// Connect PlayerWidget observer slots to Player signals
	connect( m_player, SIGNAL(fileOpened(const QString&)), this, SLOT(onPlayerFileOpened(const QString&)) );
	connect( m_player, SIGNAL(fileOpenError(const QString&)), this, SLOT(onPlayerFileOpenError(const QString&)) );
	connect( m_player, SIGNAL(fileClosed()), this, SLOT(onPlayerFileClosed()) );
	connect( m_player, SIGNAL(playbackError()), this, SLOT(onPlayerPlaybackError()) );
	connect( m_player, SIGNAL(playing()), this, SLOT(onPlayerPlaying()) );
	connect( m_player, SIGNAL(stopped()), this, SLOT(onPlayerStopped()) );
	connect( m_player, SIGNAL(positionChanged(double)), this, SLOT(onPlayerPositionChanged(double)) );
	connect( m_player, SIGNAL(lengthChanged(double)), this, SLOT(onPlayerLengthChanged(double)) );
	connect( m_player, SIGNAL(framesPerSecondChanged(double)), this, SLOT(onPlayerFramesPerSecondChanged(double)) );
	connect( m_player, SIGNAL(volumeChanged(double)), this, SLOT(onPlayerVolumeChanged(double)) );

	connect( m_player, SIGNAL(leftClicked(const QPoint&)), this, SLOT(onPlayerLeftClicked(const QPoint&)) );
	connect( m_player, SIGNAL(rightClicked(const QPoint&)), this, SLOT(onPlayerRightClicked(const QPoint&)) );
	connect( m_player, SIGNAL(doubleClicked(const QPoint&)), this, SLOT(onPlayerDoubleClicked(const QPoint&)) );

	setOverlayLine( 0 );
	onPlayerFileClosed();
	onPlayerOptionChanged( QString::null, QString::null ); // initializes the font
}

PlayerWidget::~PlayerWidget()
{
	m_player->setApplicationClosingDown();
	m_player->deinitBackend();

	delete m_fullScreenControls;
}

QToolButton* PlayerWidget::toolButton( QWidget* parent, const char* name )
{
	return (QToolButton*)parent->child( name );
}

QToolButton* PlayerWidget::createToolButton( QWidget* parent, const char* name, int size )
{
	QToolButton* toolButton = new QToolButton( parent, name );
	toolButton->setMinimumSize( size, size );
	toolButton->setAutoRaise( true );

	return toolButton;
}

void PlayerWidget::loadConfig()
{
	KConfigGroupSaver saver( app()->config(), "Player Settings" );

	onPlayerVolumeChanged( m_player->volume() );

	m_showPositionEdit = app()->config()->readBoolEntry( "EditablePositionControl", false );
	if ( m_showPositionEdit && m_player->state() > Player::Opening )
		m_positionEdit->show();
	else
		m_positionEdit->hide();
}


void PlayerWidget::saveConfig()
{
	KConfigGroupSaver saver( app()->config(), "Player Settings" );

	app()->config()->writeEntry( "EditablePositionControl", m_showPositionEdit );
}

bool PlayerWidget::fullScreenMode() const
{
	return m_fullScreenMode;
}

void PlayerWidget::setFullScreenMode( bool fullScreenMode )
{
	if ( m_fullScreenMode != fullScreenMode )
	{
		m_fullScreenMode = fullScreenMode;

		if ( m_fullScreenMode )
		{
			increaseFontSize( 10 );

			app()->mainWidget()->hide();

			m_layeredWidget->reparent( 0, 0, app()->desktop()->screenGeometry( this ).topLeft(), true );
			m_layeredWidget->showFullScreen();

			m_fullScreenControls->attach( m_layeredWidget );

			startTimer( 2000 );
		}
		else
		{
			killTimers();

			decreaseFontSize( 10 );

			app()->mainWidget()->show();

			m_layeredWidget->showNormal();
			m_layeredWidget->reparent( this, 0, QPoint( 0, 0 ), true );

			m_fullScreenControls->dettach();

			m_mainLayout->addWidget( m_layeredWidget, 0, 1 );
		}
	}
}

SubtitleLine* PlayerWidget::playingLine()
{
	return m_playingLine;
}

SubtitleLine* PlayerWidget::overlayLine()
{
	return m_overlayLine;
}

void PlayerWidget::plugActions()
{
	app()->action( ACT_STOP )->plug( toolButton( this, ACT_STOP ) );
	app()->action( ACT_PLAY_PAUSE )->plug( toolButton( this, ACT_PLAY_PAUSE ) );
	app()->action( ACT_SEEK_BACKWARDS )->plug( toolButton( this, ACT_SEEK_BACKWARDS ) );
	app()->action( ACT_SEEK_FORWARDS )->plug( toolButton( this, ACT_SEEK_FORWARDS ) );
	app()->action( ACT_SEEK_TO_PREVIOUS_LINE )->plug( toolButton( this, ACT_SEEK_TO_PREVIOUS_LINE ) );
	app()->action( ACT_SEEK_TO_NEXT_LINE )->plug( toolButton( this, ACT_SEEK_TO_NEXT_LINE ) );
	app()->action( ACT_SET_CURRENT_LINE_SHOW_TIME )->plug( toolButton( this, ACT_SET_CURRENT_LINE_SHOW_TIME ) );
	app()->action( ACT_SET_CURRENT_LINE_HIDE_TIME )->plug( toolButton( this, ACT_SET_CURRENT_LINE_HIDE_TIME ) );
	app()->action( ACT_CURRENT_LINE_FOLLOWS_VIDEO )->plug( toolButton( this, ACT_CURRENT_LINE_FOLLOWS_VIDEO ) );
	app()->action( ACT_TOGGLE_MUTED )->plug( toolButton( this, ACT_TOGGLE_MUTED ) );

	app()->action( ACT_STOP )->plug( toolButton( m_fullScreenControls, ACT_STOP ) );
	app()->action( ACT_PLAY_PAUSE )->plug( toolButton( m_fullScreenControls, ACT_PLAY_PAUSE ) );
	app()->action( ACT_SEEK_BACKWARDS )->plug( toolButton( m_fullScreenControls, ACT_SEEK_BACKWARDS ) );
	app()->action( ACT_SEEK_FORWARDS )->plug( toolButton( m_fullScreenControls, ACT_SEEK_FORWARDS ) );
	app()->action( ACT_SEEK_TO_PREVIOUS_LINE )->plug( toolButton( m_fullScreenControls, ACT_SEEK_TO_PREVIOUS_LINE ) );
	app()->action( ACT_SEEK_TO_NEXT_LINE )->plug( toolButton( m_fullScreenControls, ACT_SEEK_TO_NEXT_LINE ) );
	app()->action( ACT_TOGGLE_MUTED )->plug( toolButton( m_fullScreenControls, ACT_TOGGLE_MUTED ) );
}

void PlayerWidget::timerEvent( QTimerEvent* /*event*/ )
{
	if ( m_cursorPos != m_lastCursorPos )
		m_lastCursorPos = m_cursorPos;
	else if ( m_layeredWidget->cursor().shape() != Qt::BlankCursor )
		m_layeredWidget->setCursor( QCursor( Qt::BlankCursor ) );
}

bool PlayerWidget::eventFilter( QObject* object, QEvent* event )
{
	if ( object == m_layeredWidget )
	{
		if ( event->type() == QEvent::DragEnter )
		{
			QDragEnterEvent* dragEnterEvent = static_cast<QDragEnterEvent*>( event );
			dragEnterEvent->accept( KURLDrag::canDecode( dragEnterEvent ) );
			return true;
		}
		else if ( event->type() == QEvent::DragMove )
		{
			return true;
		}
		else if ( event->type() == QEvent::Drop )
		{
			QDropEvent* dropEvent = static_cast<QDropEvent*>( event );

			KURL::List urls;
			if ( KURLDrag::decode( dropEvent, urls ) && ! urls.isEmpty() )
			{
				for ( KURL::List::ConstIterator it = urls.begin(), end = urls.end(); it != end; ++it )
				{
					if ( (*it).protocol() != "file" )
						continue;

					app()->openVideo( *it );
					break;
				}
			}
			return true; // eat event
		}
		else if ( event->type() == QEvent::KeyPress )
		{
			// NOTE: when on full screen mode, the keyboard input is received but
			// for some reason it doesn't trigger the correct actions automatically
			// so we process the event and handley the issue ourselves.

			QKeyEvent* keyEvent = static_cast<QKeyEvent*>( event );
			int key = keyEvent->key();
			int state = keyEvent->state();
			if ( state & Qt::ControlButton )
				key |= Qt::CTRL;
			if ( state & Qt::AltButton )
				key |= Qt::ALT;
			if ( state & Qt::ShiftButton )
				key |= Qt::SHIFT;
			if ( state & Qt::MetaButton )
				key |= Qt::META;
			app()->triggerAction( QKeySequence( key ) );
			return true; // eat event
		}
		else if ( event->type() == QEvent::MouseMove )
		{
			QMouseEvent* mouseEvent = static_cast<QMouseEvent*>( event );
			m_cursorPos = mouseEvent->pos();

			if ( m_layeredWidget->cursor().shape() == Qt::BlankCursor )
				m_layeredWidget->setCursor( Qt::ArrowCursor );
		}
	}
	else if ( object == m_infoControlsGroupBox || object->parent() == m_infoControlsGroupBox )
	{
		if ( event->type() != QEvent::MouseButtonRelease )
			return QWidget::eventFilter( object, event );

		QMouseEvent* mouseEvent = static_cast<QMouseEvent*>( event );

		if ( mouseEvent->button() != Qt::RightButton )
			return QWidget::eventFilter( object, event );

		KPopupMenu popup;
		popup.insertItem( i18n( "Show editable position control" ), 0 );
		popup.setItemChecked( 0, m_showPositionEdit );
		if ( popup.exec( mouseEvent->globalPos() ) == 0 )
		{
			m_showPositionEdit = ! m_showPositionEdit;
			if ( m_showPositionEdit && m_player->state() > Player::Opening )
				m_positionEdit->show();
			else
				m_positionEdit->hide();
		}

		return true; // eat event
	}

	return QWidget::eventFilter( object, event );
}

void PlayerWidget::setSubtitle( Subtitle* subtitle )
{
	if ( m_subtitle )
	{
		disconnect( m_subtitle, SIGNAL(linesInserted(unsigned,unsigned)), this, SLOT(invalidateOverlayLine()) );
		disconnect( m_subtitle, SIGNAL(linesRemoved(unsigned,unsigned)), this, SLOT(invalidateOverlayLine()) );

		m_subtitle = 0;	// has to be set to 0 for invalidateOverlayLine

		invalidateOverlayLine();
		setPlayingLine( 0 );
	}

	m_subtitle = subtitle;

	if ( m_subtitle )
	{
		connect( m_subtitle, SIGNAL(linesInserted(unsigned,unsigned)), this, SLOT(invalidateOverlayLine()) );
		connect( m_subtitle, SIGNAL(linesRemoved(unsigned,unsigned)), this, SLOT(invalidateOverlayLine()) );
	}
}


void PlayerWidget::setTranslationMode( bool enabled )
{
	m_translationMode = enabled;

	if ( ! m_translationMode )
		setShowTranslation( false );
}

void PlayerWidget::setShowTranslation( bool showTranslation )
{
	if ( m_showTranslation != showTranslation )
	{
		m_showTranslation = showTranslation;

		invalidateOverlayLine();
		setPlayingLine( 0 );
	}
}

void PlayerWidget::increaseFontSize( int points )
{
	app()->playerConfig()->setOption( OPT_FONT_SIZE, m_textOverlay->font().pointSize() + points );
}

void PlayerWidget::decreaseFontSize( int points )
{
	app()->playerConfig()->setOption( OPT_FONT_SIZE, m_textOverlay->font().pointSize() - points );
}

void PlayerWidget::updateOverlayLine( const Time& videoPosition )
{
	if ( ! m_subtitle )
		return;

	bool seekedBackwards = m_lastCheckedTime > videoPosition;
	m_lastCheckedTime = videoPosition;

	if ( m_overlayLine )
	{
		if ( ! seekedBackwards && videoPosition <= m_overlayLine->hideTime() )
		{
			// m_overlayLine is the line to show or the next line to show
			if ( videoPosition >= m_overlayLine->showTime() ) // m_overlayLine is the line to show
			{
				m_textOverlay->setText( m_showTranslation ? m_overlayLine->secondaryText() : m_overlayLine->primaryText() );
				int styleFlags = m_overlayLine->styleFlags();
				m_textOverlay->setStyle(
					styleFlags & SubtitleLine::Bold,
					styleFlags & SubtitleLine::Italic,
					styleFlags & SubtitleLine::Underline
				);
				setPlayingLine( m_overlayLine );
			}
			return;
		}
		else
		{
			// m_overlayLine is no longer the line to show nor the next line to show
			m_textOverlay->setText( "" );

			setOverlayLine( 0 );
		}
	}

	if ( seekedBackwards || m_lastSearchedLineToShowTime > videoPosition )
	{
		// search the next line to show
		for ( SubtitleIterator it( *m_subtitle ); it.current(); ++it )
		{
			if ( videoPosition <= it.current()->hideTime() )
			{
				m_lastSearchedLineToShowTime = videoPosition;

				setOverlayLine( it.current() );

				if ( m_overlayLine->showTime() <= videoPosition && videoPosition <= m_overlayLine->hideTime() )
				{
					m_textOverlay->setText( m_showTranslation ? m_overlayLine->secondaryText() : m_overlayLine->primaryText() );
					int styleFlags = m_overlayLine->styleFlags();
					m_textOverlay->setStyle(
						styleFlags & SubtitleLine::Bold,
						styleFlags & SubtitleLine::Italic,
						styleFlags & SubtitleLine::Underline
					);
					setPlayingLine( m_overlayLine );
				}
				return;
			}
		}
		setPlayingLine( 0 );
	}
}

void PlayerWidget::invalidateOverlayLine()
{
	m_textOverlay->setText( QString::null );

	setOverlayLine( 0 );

	updateOverlayLine( (long)(m_player->position()*1000) );
}

void PlayerWidget::setOverlayLine( SubtitleLine* line )
{
	if ( m_overlayLine )
	{
		disconnect( m_overlayLine, SIGNAL(showTimeChanged(const Time&)), this, SLOT(invalidateOverlayLine()) );
		disconnect( m_overlayLine, SIGNAL(hideTimeChanged(const Time&)), this, SLOT(invalidateOverlayLine()) );
	}

	m_overlayLine = line;

	if ( m_overlayLine )
	{
		connect( m_overlayLine, SIGNAL(showTimeChanged(const Time&)), this, SLOT(invalidateOverlayLine()) );
		connect( m_overlayLine, SIGNAL(hideTimeChanged(const Time&)), this, SLOT(invalidateOverlayLine()) );
	}
	else
		m_lastSearchedLineToShowTime = Time::MAX_MSECONDS;
}

void PlayerWidget::setPlayingLine( SubtitleLine* line )
{
	if ( ! line || m_playingLine != line )
	{
		m_playingLine = line;
		emit playingLineChanged( m_playingLine );
	}
}


void PlayerWidget::onVolumeSliderValueChanged( int value )
{
	if ( m_updatePlayerVolume )
	{
		m_updateVolumeControls = false;
		if ( sender() == m_fsVolumeSlider )
			m_player->setVolume( value );
		else
			m_player->setVolume( 100 - value );
		m_updateVolumeControls = true;
	}
}

void PlayerWidget::onSeekSliderPressed()
{
	m_updatePositionControls = 0;
}

void PlayerWidget::onSeekSliderReleased()
{
	m_updatePositionControls = MAGIC_NUMBER;
}

void PlayerWidget::onSeekSliderValueChanged( int value )
{
	if ( m_updateVideoPosition )
	{
		m_updatePositionControls = MAGIC_NUMBER;
		m_player->seek( m_player->length()*value/1000.0, true );
	}
}

void PlayerWidget::onSeekSliderMoved( int value )
{
	m_player->seek( m_player->length()*value/1000.0, false );

	Time time( (long)(m_player->length()*value) );

	m_positionLabel->setText( time.toString() );
	m_fsPositionLabel->setText( time.toString() + m_lengthString );

	if ( m_showPositionEdit )
		m_positionEdit->setValue( time.toMillis() );
}

void PlayerWidget::onPositionEditValueChanged( int position )
{
	if ( m_positionEdit->hasFocus() )
	{
		m_updatePositionControls = MAGIC_NUMBER;
		m_player->seek( position/1000.0, true );
	}
}

void PlayerWidget::onPlayerOptionChanged( const QString& option, const QString& value )
{
	if ( option == OPT_PLAYER_BACKEND )
	{
		m_player->setActiveBackend( value, m_layeredWidget );
	}
	else
	{
		m_textOverlay->setColor( app()->playerConfig()->optionAsColor( OPT_FONT_COLOR ) );

		QFont font( m_textOverlay->font() );
		font.setFamily( app()->playerConfig()->option( OPT_FONT_FAMILY ) );
		font.setPointSize( app()->playerConfig()->optionAsInt( OPT_FONT_SIZE ) );
		m_textOverlay->setFont( font );
	}
}

void PlayerWidget::onPlayerFileOpened( const QString& /*filePath*/ )
{
	m_infoControlsGroupBox->setEnabled( true );

	if ( m_showPositionEdit )
		m_positionEdit->show();

	m_positionLabel->setText( i18n( "<i>Unknown</i>" ) );
	m_lengthLabel->setText( i18n( "<i>Unknown</i>" ) );
	m_fpsLabel->setText( i18n( "<i>Unknown</i>" ) );

	m_lengthString = UNKNOWN_LENGTH_STRING;

	m_fsPositionLabel->setText( Time().toString() + m_lengthString );
}

void PlayerWidget::onPlayerFileOpenError( const QString& filePath )
{
	KMessageBox::sorry( this, i18n( "<qt>There was an error opening file<br/>%1</qt>" ).arg( filePath ) );
}

void PlayerWidget::onPlayerFileClosed()
{
	m_lastCheckedTime = 0;

	m_infoControlsGroupBox->setEnabled( false );

	m_positionEdit->hide();
	m_positionEdit->setValue( 0 );

 	m_positionLabel->setText( QString() );
	m_lengthLabel->setText( QString() );
	m_fpsLabel->setText( QString() );

	m_lengthString = UNKNOWN_LENGTH_STRING;

	m_fsPositionLabel->setText( Time().toString() + m_lengthString );

	m_seekSlider->setEnabled( false );
	m_fsSeekSlider->setEnabled( false );
}

void PlayerWidget::onPlayerPlaybackError()
{
	KMessageBox::sorry( this, i18n( "Unexpected error while playing file." ) );
	//onPlayerFileClosed();
}

void PlayerWidget::onPlayerPlaying()
{
	m_seekSlider->setEnabled( true );
	m_fsSeekSlider->setEnabled( true );

}

void PlayerWidget::onPlayerPositionChanged( double seconds )
{
	if ( m_updatePositionControls > 0 )
	{
		if ( seconds >= 0 )
		{
			Time videoPosition( (long)(seconds*1000) );

			m_positionLabel->setText( videoPosition.toString() );
			m_fsPositionLabel->setText( videoPosition.toString() + m_lengthString );

			if ( m_showPositionEdit && ! m_positionEdit->hasFocus() )
				m_positionEdit->setValue( videoPosition.toMillis() );

			updateOverlayLine( videoPosition );

			int sliderValue = (int)((seconds/m_player->length())*1000);

			m_updateVideoPosition = false;
			m_seekSlider->setValue( sliderValue );
			m_fsSeekSlider->setValue( sliderValue );
			m_updateVideoPosition = true;
		}
		else
		{
			m_positionLabel->setText( i18n( "<i>Unknown</i>" ) );
			m_fsPositionLabel->setText( Time().toString() + m_lengthString );
		}
	}
 	else if ( m_updatePositionControls < 0 )
		m_updatePositionControls += 2;
}

void PlayerWidget::onPlayerLengthChanged( double seconds )
{
	if ( seconds > 0 )
	{
		m_lengthLabel->setText( Time( (long)(seconds*1000) ).toString() );
		m_lengthString = " / " + m_lengthLabel->text() + " ";
	}
	else
	{
		m_lengthLabel->setText( i18n( "<i>Unknown</i>" ) );
		m_lengthString = UNKNOWN_LENGTH_STRING;
	}
}

void PlayerWidget::onPlayerFramesPerSecondChanged( double fps )
{
	m_fpsLabel->setText( fps > 0 ? QString::number( fps, 'f', 3 ) : i18n( "<i>Unknown</i>" ) );
}

void PlayerWidget::onPlayerStopped()
{
	onPlayerPositionChanged( 0 );

	m_seekSlider->setEnabled( false );
	m_fsSeekSlider->setEnabled( false );

	setPlayingLine( 0 );
}

void PlayerWidget::onPlayerVolumeChanged( double volume )
{
	if ( m_updateVolumeControls )
	{
		m_updatePlayerVolume = false;
		m_volumeSlider->setValue( 100 - (int)(volume + 0.5) );
		m_fsVolumeSlider->setValue( (int)(volume + 0.5) );
		m_updatePlayerVolume = true;
	}
}

void PlayerWidget::onPlayerLeftClicked( const QPoint& /*point*/ )
{
	m_player->togglePlayPaused();
}

void PlayerWidget::onPlayerRightClicked( const QPoint& point )
{
	static KPopupMenu* popup = new KPopupMenu( this );

	popup->clear();

	app()->action( ACT_OPEN_VIDEO )->plug( popup );
	app()->action( ACT_CLOSE_VIDEO )->plug( popup );

	popup->insertSeparator();

	app()->action( ACT_TOGGLE_FULL_SCREEN )->plug( popup );

	popup->insertSeparator();

	app()->action( ACT_STOP )->plug( popup );
	app()->action( ACT_PLAY_PAUSE )->plug( popup );
	app()->action( ACT_SEEK_BACKWARDS )->plug( popup );
	app()->action( ACT_SEEK_FORWARDS )->plug( popup );

	popup->insertSeparator();

	app()->action( ACT_SEEK_TO_PREVIOUS_LINE )->plug( popup );
	app()->action( ACT_SEEK_TO_NEXT_LINE )->plug( popup );

	popup->insertSeparator();

	app()->action( ACT_TOGGLE_MUTED )->plug( popup );
	app()->action( ACT_DECREASE_VOLUME )->plug( popup );
	app()->action( ACT_INCREASE_VOLUME )->plug( popup );
	app()->action( ACT_SET_ACTIVE_AUDIO_STREAM )->plug( popup );

	popup->insertSeparator();

	app()->action( ACT_DECREASE_SUBTITLE_FONT )->plug( popup );
	app()->action( ACT_INCREASE_SUBTITLE_FONT )->plug( popup );

	if ( m_translationMode )
		app()->action( ACT_SET_ACTIVE_SUBTITLE_STREAM )->plug( popup );

	// NOTE do not use popup->exec() here!!! it freezes the application
	// when using the mplayer backend. i think it's related to the fact
	// that exec() creates a different event loop and the mplayer backend
	// depends on the main loop for catching synchronization signals
	popup->popup( point );
}

void PlayerWidget::onPlayerDoubleClicked( const QPoint& /*point*/ )
{
	app()->toggleFullScreenMode();
}

void PlayerWidget::onPlayerBackendInitialized()
{
	// NOTE when the player backend is initialized the video widget
	// is created in front of the text overlay, so the have to raise
	// it to make it visible again
	m_textOverlay->raise();
}

#include "playerwidget.moc"
