/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kactionext.h"

#include <qtoolbutton.h>

#include <kapplication.h>
#include <kiconloader.h>

KActionExt::KActionExt( KActionCollection* parent, const char* name ):
	KAction( parent, name )
{
}

KActionExt::KActionExt( const KShortcut& defaultShortcut, KActionCollection* parent, const char* name ):
	KAction( QString::null, defaultShortcut, 0, 0, parent, name )
{
}

int KActionExt::plug( QWidget* widget, int index )
{
	if ( ! ::qt_cast<QToolButton*>( widget ) )
		return KAction::plug( widget, index );

	QToolButton* toolButton = static_cast<QToolButton*>( widget );

	unplug( toolButton );

	toolButton->setTextLabel( text() );
	toolButton->setIconSet( iconSet() );
	toolButton->setEnabled( isEnabled() );

	connect( this, SIGNAL(enabled(bool)), toolButton, SLOT(setEnabled(bool)) );
	connect( toolButton, SIGNAL(clicked()), this, SLOT(slotActivated()) );

	m_toolButtons.append( toolButton );

	return 0;
}

void KActionExt::unplug( QWidget* widget )
{
	if ( ! ::qt_cast<QToolButton*>( widget ) )
	{
		KAction::unplug( widget );
		return;
	}

	QToolButton* toolButton = static_cast<QToolButton*>( widget );

	if ( m_toolButtons.remove( toolButton ) )
	{
		disconnect( this, SIGNAL(enabled(bool)), toolButton, SLOT(setEnabled(bool)) );
		disconnect( toolButton, SIGNAL(clicked()), this, SLOT(slotActivated()) );
	}
}

void KActionExt::setText( const QString& text )
{
	KAction::setText( text );

	for ( QValueListConstIterator<QToolButton*> it = m_toolButtons.begin(), end = m_toolButtons.end(); it != end; ++it )
		(*it)->setTextLabel( text );
}

void KActionExt::setIconSet( const QIconSet& iconSet )
{
	KAction::setIconSet( iconSet );

	for ( QValueListConstIterator<QToolButton*> it = m_toolButtons.begin(), end = m_toolButtons.end(); it != end; ++it )
		(*it)->setIconSet( iconSet );
}

void KActionExt::setIcon( const QString& icon )
{
	KAction::setIcon( icon );

	QIconSet iconSet = kapp->iconLoader()->loadIconSet( icon, KIcon::Small, 16 );
	for ( QValueListConstIterator<QToolButton*> it = m_toolButtons.begin(), end = m_toolButtons.end(); it != end; ++it )
		(*it)->setIconSet( iconSet );
}

#include "kactionext.moc"
