/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "ktoggleactionext.h"

#include <qtoolbutton.h>

#include <kapplication.h>
#include <kiconloader.h>

KToggleActionExt::KToggleActionExt( KActionCollection* parent, const char* name ):
	KToggleAction( parent, name ),
	m_offText( QString::null ),
	m_onText( QString::null )
{
}

KToggleActionExt::KToggleActionExt( const KShortcut& defaultShortcut, KActionCollection* parent, const char* name ):
	KToggleAction( QString::null, defaultShortcut, parent, name ),
	m_offText( QString::null ),
	m_onText( QString::null )
{
}

int KToggleActionExt::plug( QWidget* widget, int index )
{
	if ( ! ::qt_cast<QToolButton*>( widget ) )
		return KToggleAction::plug( widget, index );

	QToolButton* toolButton = static_cast<QToolButton*>( widget );

	unplug( toolButton );

	toolButton->setToggleButton( true );
	toolButton->setTextLabel( KToggleAction::text() );
	toolButton->setOn( isChecked() );
	toolButton->setIconSet( iconSet() );
	toolButton->setEnabled( isEnabled() );

	connect( this, SIGNAL(toggled(bool)), toolButton, SLOT(setOn(bool)) );
	connect( this, SIGNAL(enabled(bool)), toolButton, SLOT(setEnabled(bool)) );
	connect( toolButton, SIGNAL(clicked()), this, SLOT(slotActivated()) );

	m_toolButtons.append( toolButton );

	return 0;
}

void KToggleActionExt::unplug( QWidget* widget )
{
	if ( ! ::qt_cast<QToolButton*>( widget ) )
	{
		KToggleAction::unplug( widget );
		return;
	}

	QToolButton* toolButton = static_cast<QToolButton*>( widget );

	if ( m_toolButtons.remove( toolButton ) )
	{
		disconnect( this, SIGNAL(toggled(bool)), toolButton, SLOT(setOn(bool)) );
		disconnect( this, SIGNAL(enabled(bool)), toolButton, SLOT(setEnabled(bool)) );
		disconnect( toolButton, SIGNAL(clicked()), this, SLOT(slotActivated()) );
	}
}

void KToggleActionExt::setChecked( bool checked )
{
	if ( isChecked() != checked )
	{
		KToggleAction::setChecked( checked );

		for ( QValueListConstIterator<QToolButton*> it = m_toolButtons.begin(), end = m_toolButtons.end(); it != end; ++it )
			(*it)->setOn( checked );

		setText( m_offText, m_onText );
	}
}

void KToggleActionExt::setText( const QString& text )
{
	setText( text, text );
}

void KToggleActionExt::setText( const QString& offText, const QString& onText )
{
	m_offText = offText;
	m_onText = onText;

	QString text = isChecked() ? m_onText : m_offText;

	KToggleAction::setText( text );

	for ( QValueListConstIterator<QToolButton*> it = m_toolButtons.begin(), end = m_toolButtons.end(); it != end; ++it )
		(*it)->setTextLabel( text );
}

void KToggleActionExt::setIconSet( const QIconSet& iconSet )
{
	KToggleAction::setIconSet( iconSet );

	for ( QValueListConstIterator<QToolButton*> it = m_toolButtons.begin(), end = m_toolButtons.end(); it != end; ++it )
		(*it)->setIconSet( iconSet );
}

void KToggleActionExt::setIcon( const QString& icon )
{
	KToggleAction::setIcon( icon );

	QIconSet iconSet( kapp->iconLoader()->loadIconSet( icon, KIcon::Small, 16 ) );
	for ( QValueListConstIterator<QToolButton*> it = m_toolButtons.begin(), end = m_toolButtons.end(); it != end; ++it )
		(*it)->setIconSet( iconSet );
}
