#ifndef FIND_H
#define FIND_H

/***************************************************************************
*   Copyright (C) 2007-2008 by Sergio Pistone                             *
*   sergio_pistone@yahoo.com.ar                                           *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <qobject.h>
#include <ranges.h>
#include <subtitle.h>
#include <subtitleline.h>
#include <subtitleiterator.h>

class KFind;
class KFindDialog;

namespace SubtitleComposer
{
	class Find : public QObject
	{
		Q_OBJECT

		public:

			Find( QWidget* parent );
			~Find();

			QWidget* parentWidget();

		public slots:

			void setSubtitle( Subtitle* subtitle=0 );
			void setTranslationMode( bool enabled );

			void find( const RangeList& selectionRanges, unsigned currentIndex, bool findBackwards=false );
			bool findNext();
			bool findPrevious();

		signals:

			void found( SubtitleLine* line, bool primary, int startIndex, int endIndex );

		private slots:

			void invalidate();

			void onLinePrimaryTextChanged( const QString& text );
			void onLineSecondaryTextChanged( const QString& text );

			void onHighlight( const QString& text, int matchingIndex, int matchedLength );

		private:

			void advance();

		private:

			Subtitle* m_subtitle;
			bool m_translationMode;
			bool m_feedPrimaryNext;

			KFind* m_find;
			KFindDialog* m_dialog;
			SubtitleIterator* m_iterator;
			SubtitleLine* m_dataLine;
			bool m_found;
			int m_firstIndex;
	};
}

#endif
