/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "finderror.h"

#include <kdebug.h>
#include <klocale.h>

#include <kmessagebox.h>

using namespace SubtitleComposer;

FindError::FindError( QWidget* parent ):
	QObject( parent ),
	m_subtitle( 0 ),
	m_iterator( 0 )
{
}

FindError::~FindError()
{
	setSubtitle( 0 );
}

void FindError::invalidate()
{
	delete m_iterator;
	m_iterator = 0;
}

QWidget* FindError::parentWidget()
{
	return static_cast<QWidget*>( parent() );
}

void FindError::setSubtitle( Subtitle* subtitle )
{
	if ( m_subtitle )
	{
		disconnect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ), this, SLOT( invalidate() ) );
		disconnect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ), this, SLOT( invalidate() ) );
	}

	m_subtitle = subtitle;

	invalidate();

	if ( m_subtitle )
	{
		connect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ), this, SLOT( invalidate() ) );
		connect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ), this, SLOT( invalidate() ) );
	}
}

void FindError::find( unsigned currentIndex, bool backwards )
{
	if ( ! m_iterator )
	{
		if ( ! m_subtitle || ! m_subtitle->linesCount() )
			return;

		m_iterator = new SubtitleIterator( *m_subtitle );
	}

	m_iterator->toIndex( currentIndex );
	m_firstIndex = m_iterator->index();


	bool foundError;

	do
	{
		if ( ! advance( backwards ) )
			break;

		foundError = m_iterator->current()->errorFlags() || m_iterator->current()->marked();
		if ( foundError )
			emit found( m_iterator->current() );
	}
	while ( ! foundError );

	if ( ! foundError )
		KMessageBox::sorry( parentWidget(), i18n( "No line with errors found!" ), i18n( "Find Error" ) );
}


bool FindError::advance( bool backwards )
{
	if ( backwards )
		--(*m_iterator);
	else
		++(*m_iterator);

	if ( m_firstIndex == m_iterator->index() ||
		 (backwards ?
			(m_firstIndex == m_iterator->lastIndex() && m_iterator->index() == SubtitleIterator::BEHIND_FIRST) :
			(m_firstIndex == m_iterator->firstIndex() && m_iterator->index() == SubtitleIterator::AFTER_LAST)) )
		return false;

	if ( m_iterator->index() < 0 )
	{
		if ( backwards )
			m_iterator->toLast();
		else
			m_iterator->toFirst();

		if ( KMessageBox::warningContinueCancel(
				parentWidget(),
				backwards ?
					i18n( "Beginning of subtitle reached.\nContinue from the end?" ) :
					i18n( "End of subtitle reached.\nContinue from the beginning?" ),
				i18n( "Find Error" )
				) != KMessageBox::Continue )
			return false;
	}

	return true;
}

#include "finderror.moc"
