#ifndef GSTPLAYERBACKEND_H
#define GSTPLAYERBACKEND_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *   based on Kaffeine by Jürgen Kofler                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <playerbackend.h>
#include <appconfiggroup.h>
#include <appconfiggroupwidget.h>

#include <qwidget.h>
#include <qstring.h>

#include <gst/gst.h>

class QTimer;

namespace SubtitleComposer
{
	class GstPlayerBackend : public PlayerBackend
	{
		Q_OBJECT

		public:

			GstPlayerBackend();
			virtual ~GstPlayerBackend();

			virtual AppConfigGroupWidget* newAppConfigGroupWidget( QWidget* parent );

		protected:

			virtual VideoWidget* _initialize( QWidget* videoWidgetParent );
			virtual void _finalize();
			void __finalize();

			static AppConfigGroup defaultConfigGroup();

			virtual bool _openFile( const QString& filePath, bool& playingAfterCall );
			bool __openFile( const QString& filePath, bool play );
			virtual void _closeFile();

			virtual bool _play();
			virtual bool _pause();
			virtual bool _seek( double seconds, bool accurate );
			virtual bool _stop();

			virtual bool _setActiveAudioStream( int audioStream, bool& supported );

			virtual bool _setVolume( double volume );

		protected:

			bool initializeGStreamer();
			bool createPlaybin();
			void deletePlaybin();

			bool changeState( GstState state, unsigned timeout=0 );

			GList* streamInfoForType( const gchar* typestr );
			void updateVideoData();
			void updateAudioData();
			void hideSubtitles();

			void updatePosition();

		protected slots:

			void onBusTimerTimeout();

		protected:

			GstElement* m_play;
			GstElement* m_videoSink;
			GstElement* m_audioSink;

			GstBus* m_bus;
			QTimer* m_busTimer;

			bool m_updatePosition;
			guint64 m_length;

			bool m_updateMediaData;
	};
}

#endif

