#ifndef MPLAYERBACKEND_H
#define MPLAYERBACKEND_H

/***************************************************************************
 *   smplayer, GUI front-end for mplayer.                                  *
 *   Copyright (C) 2007 Ricardo Villalba <rvm@escomposlinux.org>           *
 *                                                                         *
 *   modified for inclusion in Subtitle Composer                           *
 *   Copyright (C) 2007 by Sergio Pistone <sergio_pistone@yahoo.com.ar>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <playerbackend.h>
#include <appconfiggroup.h>
#include <appconfiggroupwidget.h>
#include "mediadata.h"

#include <qwidget.h>
#include <qstring.h>

namespace SubtitleComposer
{
	class MPlayerProcess;
	class MPlayerVideoWidget;

	class MPlayerBackend : public PlayerBackend
	{
		Q_OBJECT

		public:

			MPlayerBackend();
			~MPlayerBackend();

			virtual AppConfigGroupWidget* newAppConfigGroupWidget( QWidget* parent );

		protected:

			virtual VideoWidget* _initialize( QWidget* videoWidgetParent );
			virtual void _finalize();
			void __finalize();

			static AppConfigGroup defaultsConfigGroup();

			virtual bool _openFile( const QString& filePath, bool& playingAfterCall );
			virtual void _closeFile();

			virtual bool _play();
			virtual bool _pause();
			virtual bool _seek( double seconds, bool accurate );
			virtual bool _stop();

			virtual bool _setActiveAudioStream( int audioStream, bool& supported );

			virtual bool _setVolume( double volume );

		protected slots:

			void onMediaDataLoaded();
			void onPlayingReceived();
			void onPausedReceived();
			void onProcessExited();
			void onPositionReceived( double seconds );

		protected:

			void setupProcessArgs( const QString& filePath );

		protected:

			MPlayerProcess* m_process;

			double m_position;
			bool m_reportUpdates;
	};
}

#endif
