#ifndef MPLAYERPROCESS_H
#define MPLAYERPROCESS_H

/***************************************************************************
 *   smplayer, GUI front-end for mplayer.                                  *
 *   Copyright (C) 2007 Ricardo Villalba <rvm@escomposlinux.org>           *
 *                                                                         *
 *   modified for inclusion in Subtitle Composer                           *
 *   Copyright (C) 2007 by Sergio Pistone <sergio_pistone@yahoo.com.ar>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <appconfiggroup.h>
#include "mediadata.h"

#include <qprocess.h>
#include <qobject.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qcstring.h>
#include <qregexp.h>
#include <qtimer.h>

namespace SubtitleComposer
{
	class MPlayerProcess : public QProcess
	{
		Q_OBJECT

		friend class MPlayer;

		public:

			MPlayerProcess( const AppConfigGroup& configGroup, QObject* parent=0 );
			~MPlayerProcess();

			bool waitForFinished( int msecs=30000 );

			const MediaData& mediaData();

			/** returns false only if the executable couldn't be found (in which case the process could not be started) */
			bool start( const QString& filePath, int winId, int audioStream=-1, int audioStreamCount=1 );

			void sendTogglePause();
			void sendSeek( double seconds );
			void sendFastSeek( double seconds );

			void sendToggleMute();
			void sendVolume( double volume );
			void sendAudioStream( int audioStream );

			void sendQuit();

		signals:

			void mediaDataLoaded();
			void playingReceived();
			void pausedReceived();
			void positionReceived( double seconds );

		private slots:

			void onReadyReadStandardOutput();
			void onWroteToStdin();
			void onTimeout();

		private:

			/// indicates in wich state the player is left after the execution
			/// of a command: playing, paused or the same as before.
			typedef enum { Playing, Pausing, PausingKeep } CommandMode;

			void sendCommand( const char* cmd, CommandMode mode, bool block );
			void sendCommand( const QString& cmd, CommandMode mode, bool block );
			void queueCommand( const QString& cmd, MPlayerProcess::CommandMode mode );

			void parseLine( const QString& line );

		private:

			const AppConfigGroup& m_configGroup;
			MediaData m_mediaData;

			QStringList m_commandsQueue;
			QTimer m_commandsQueueTimer;

			QCString m_incompleteLine;

			bool m_isMediaDataLoaded;
			bool m_isPaused;
			bool m_emitPlaying;

			QRegExp m_positionRegExp;
			QRegExp m_videoFrameRegExp;
			QRegExp m_generalTagRegExp;
			QRegExp m_audioTagRegExp;
			QRegExp m_pausedTagRegExp;
	};
}

#endif
