/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "videowidget.h"

#include <qapplication.h>
#include <qdesktopwidget.h>

#include <qpainter.h>
#include <qevent.h>

#include <kdebug.h>

using namespace SubtitleComposer;

VideoWidget::VideoWidget( QWidget* parent ):
	QWidget( parent ),
	m_videoLayer( new QWidget( this ) ),
	m_videoWidth( 0 ),
	m_videoHeight( 0 ),
	m_videoAspect( 0 )
{
	m_videoLayer->hide();

	init( true );
}

VideoWidget::VideoWidget( QWidget* videoLayer, QWidget* parent ):
	QWidget( parent ),
	m_videoLayer( videoLayer ),
	m_videoWidth( 0 ),
	m_videoHeight( 0 ),
	m_videoAspect( 0 )
{
	m_videoLayer->reparent( this, 0, QPoint( 0, 0 ) );
	m_videoLayer->hide();

	init( true );
}

void VideoWidget::init( bool /*setVideoLayerAttributes*/ )
{
	/* Can't set the widget attributes in Qt3
	if ( setVideoLayerAttributes )
	{
		m_videoLayer->setFocusPolicy( QWidget::NoFocus );
		m_videoLayer->setWFlags( Qt::WStaticContents|Qt::WNoAutoErase );
	}*/

	setFocusPolicy( QWidget::NoFocus );
	setWFlags( Qt::WStaticContents|Qt::WNoAutoErase );
}

VideoWidget::~VideoWidget()
{
}

QWidget* VideoWidget::videoLayer()
{
	return m_videoLayer;
}

void VideoWidget::setVideoResolution( int width, int height, double displayAspect )
{
	m_videoWidth = width;
	m_videoHeight = height;
	m_videoAspect = displayAspect;

	updateVideoLayerGeometry();
}

void VideoWidget::setMouseTracking( bool enable )
{
	QWidget::setMouseTracking( enable );
	m_videoLayer->setMouseTracking( enable );
}

void VideoWidget::paintEvent( QPaintEvent* e )
{
	QPainter( this ).fillRect( e->rect(), Qt::black );
}

void VideoWidget::resizeEvent( QResizeEvent* )
{
	updateVideoLayerGeometry();
}

void VideoWidget::mouseReleaseEvent( QMouseEvent* e )
{
	if ( e->button() == Qt::LeftButton )
	{
		e->accept();
		emit leftClicked( e->globalPos() );
	}
	else if ( e->button() == Qt::RightButton )
	{
		e->accept();
		emit rightClicked( e->globalPos() );
	}
	else
		e->ignore();
}

void VideoWidget::mouseDoubleClickEvent( QMouseEvent* e )
{
	e->accept();
	emit doubleClicked( e->globalPos() );
}

void VideoWidget::wheelEvent( QWheelEvent * e )
{
	e->accept();
	if ( e->delta() >= 0 )
		emit wheelUp();
	else
		emit wheelDown();
}


QSize VideoWidget::desktopSize()
{
	QRect rect = QApplication::desktop()->screenGeometry( this );
	return QSize( rect.width(), rect.height() );
}

double VideoWidget::desktopAspectRatio()
{
	QSize size = desktopSize();
	return (double)size.width() / size.height() ;
}

void VideoWidget::updateVideoLayerGeometry()
{
	QSize frameSize = size();

	int videoLayerX = 0;
	int videoLayerY = 0;
	int videoLayerWidth = frameSize.width();
	int videoLayerHeight = frameSize.height();

	if ( m_videoAspect > 0.0 && m_videoWidth != 0 && m_videoHeight != 0 )
	{
		if ( m_videoAspect >= ((double)videoLayerWidth / videoLayerHeight) )
		{
			videoLayerHeight = (int)(videoLayerWidth / m_videoAspect);
			videoLayerY = (frameSize.height() - videoLayerHeight) / 2;
		}
		else
		{
			videoLayerWidth = (int)(videoLayerHeight * m_videoAspect);
			videoLayerX = (frameSize.width() - videoLayerWidth) / 2;
		}
	}

	m_videoLayer->setGeometry( videoLayerX, videoLayerY, videoLayerWidth, videoLayerHeight );
}

QSize VideoWidget::sizeHint() const
{
	return QSize( m_videoWidth, m_videoHeight );
}

QSize VideoWidget::minimumSizeHint() const
{
	return QSize( 0, 0 );
}
