#ifndef PLAYERVIDEOWIDGET_H
#define PLAYERVIDEOWIDGET_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <qwidget.h>
#include <qpoint.h>
#include <qsize.h>

class QResizeEvent;
class QPaintEvent;
class QMouseEvent;
class QWheelEvent;

namespace SubtitleComposer
{
	class VideoWidget : public QWidget
	{
		Q_OBJECT

		public:

			VideoWidget( QWidget* videoLayer, QWidget* parent );
			VideoWidget( QWidget* parent );
			~VideoWidget();

			QWidget* videoLayer();

			/// if displayAspect is <= 0, it's ignored (the displayAspect becomes width/height)
			void setVideoResolution( int width, int height, double displayAspect=0.0 );

			virtual QSize sizeHint() const;
			virtual QSize minimumSizeHint() const;

			double desktopAspectRatio();
			QSize desktopSize();

			void updateVideoLayerGeometry();

		public slots:

			virtual void setMouseTracking( bool enable );

		signals:

			void doubleClicked( const QPoint& point );
			void rightClicked( const QPoint& point );
			void leftClicked( const QPoint& point );
			void wheelUp();
			void wheelDown();

		protected:

			void init( bool setVideoLayerAttributes );

			virtual void paintEvent( QPaintEvent* e );
			virtual void resizeEvent( QResizeEvent* e);

			virtual void mouseReleaseEvent( QMouseEvent* e );
			virtual void mouseDoubleClickEvent( QMouseEvent* e );
			virtual void wheelEvent( QWheelEvent* e );

		protected:

			QWidget* m_videoLayer;

			int m_videoWidth;
			int m_videoHeight;
			double m_videoAspect;
	};
}

#endif
