/**************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ATTACHABLE_WIDGET_H
#define ATTACHABLE_WIDGET_H

#include <qwidget.h>
#include <qtoolbar.h>

class AttachableWidget : public QWidget
{
	Q_OBJECT

	public:

		typedef enum { Top, Bottom } Place;

		AttachableWidget( bool animated=true, unsigned stepDuration=4 );
		~AttachableWidget();

		bool isAnimated() const;
		void setAnimated( bool animated );

		unsigned animStepDuration() const;
		void setAnimStepDuration( unsigned msecs );

		/// Show the floating widget over the specified widget.
		void attach( QWidget* widget, Place place=Bottom );
		void dettach();

		virtual bool eventFilter( QObject* object, QEvent* event );

	protected:

		virtual void timerEvent( QTimerEvent* event );

		void toggleVisible( bool visible );

	private:

		typedef enum { Upward, Downward } Direction;

		QWidget* m_widget;
		Place m_place;

		bool m_animated;
		bool m_hiding;

		int m_ignoreMouseTID;
		bool m_ignoreMouse;

		unsigned m_animStepDuration;
		int m_animTID;
		int m_animCurrentY;
		int m_animFinalY;
		Direction m_animDirection;
};

#endif
