/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "layeredwidget.h"

#include <qobjectlist.h>

LayeredWidget::LayeredWidget( QWidget* parent ):
	QWidget( parent, 0, 0 )
{
}

void LayeredWidget::setWidgetMode( QWidget* widget, LayeredWidget::Mode mode )
{
	m_ignoredWidgets.removeRef( widget );
	if ( mode == Ignore )
		m_ignoredWidgets.append( widget );
}

void LayeredWidget::setMouseTracking( bool enable )
{
	QWidget::setMouseTracking( enable );

	QObjectList objects( *children() );
	for ( QObject* object = objects.first(); object; object = objects.next() )
	{
		try
		{
			QWidget* widget = dynamic_cast<QWidget*>( object );
			if ( widget )
				widget->setMouseTracking( enable );
		}
		catch( const std::bad_cast& )
		{
		}
	}
}

void LayeredWidget::resizeEvent( QResizeEvent* )
{
	QObjectList objects( *children() );
	for ( QObject* object = objects.first(); object; object = objects.next() )
	{
		try
		{
			QWidget* widget = dynamic_cast<QWidget*>( object );
			if ( widget && m_ignoredWidgets.findRef( widget ) < 0 )
				widget->resize( size() );
		}
		catch( const std::bad_cast& )
		{
		}
	}
}
