/***************************************************************************
 *   smplayer, GUI front-end for mplayer.                                  *
 *   Copyright (C) 2007 Ricardo Villalba <rvm@escomposlinux.org>           *
 *                                                                         *
 *   modified for inclusion in Subtitle Composer                           *
 *   Copyright (C) 2007 by Sergio Pistone <sergio_pistone@yahoo.com.ar>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "pointingslider.h"

#if QT_VERSION > 0x040000
	#include "qt3_4_compat.h"
	#include <QApplication>
	#include <QStyle>
	#include <QLayout>
#else
	#include <qlayout.h>
#endif


PointingSlider::PointingSlider( QWidget* parent ):
	QSlider( parent, 0 )
{
}

PointingSlider::PointingSlider( Orientation orientation, QWidget* parent ):
	QSlider( orientation, parent, 0 )
{
}

PointingSlider::PointingSlider( int minValue, int maxValue, int pageStep, int value, Orientation orientation, QWidget* parent ):
	QSlider( minValue, maxValue, pageStep, value, orientation, parent, 0 )
{
}

PointingSlider::~PointingSlider()
{
}

// The code from the following function is from Javier Díaz,
// taken from a post in the Qt-interest mailing list.
void PointingSlider::mousePressEvent( QMouseEvent* e )
{
#if QT_VERSION < 0x040000
	if ( e->button() == Qt::LeftButton )
	{
		if ( ! sliderRect().contains( e->pos() ) )
		{
			int value;
			if ( orientation() == Qt::Horizontal )
				value = QRangeControl::valueFromPosition( e->x(), width() );
			else
				value = QRangeControl::valueFromPosition( e->y(), height() );

			QSlider::setValue( value );
			emit sliderMoved( value );
		}
		else
			QSlider::mousePressEvent( e );
	}
#else
	int range = maximum()-minimum();
	int pos;
	if ( orientation() == Qt::Horizontal )
		pos = (e->x() * range) / width();
	else
		pos = (e->y() * range) / height();

	// Calculate how many positions takes the slider handle
	int metric = qApp->style()->pixelMetric( QStyle::PM_SliderLength );
	double one_tick_pixels = (double) width() / range;
	int slider_handle_positions = metric / one_tick_pixels;

	if ( abs(pos - QSlider::value()) > slider_handle_positions )
	{
		setValue( pos );
		emit sliderMoved( pos );
	}
	else
		QSlider::mousePressEvent( e );
#endif
}
