/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "textoverlaywidget.h"

#include <qpixmap.h>
#include <qbitmap.h>
#include <qpainter.h>
#include <qimage.h>

TextOverlayWidget::TextOverlayWidget( QWidget* parent ):
	QWidget( parent, 0, 0 ),
	m_text(),
	m_font(),
	m_pen(),
	m_transColor( 0, 0, 0 ),
	m_transRGB( qRgb( 0, 0, 0 ) ),
	m_dirty( true )
{
// 	m_font.setBold( true );
	m_font.setPointSize( 15 );
	m_pen.setColor( Qt::yellow );
}

QString TextOverlayWidget::text()
{
	return m_text;
}

void TextOverlayWidget::setText( const QString& text )
{
	if ( m_text != text )
	{
		m_text = text;
		m_dirty = true;
		update();
	}
}

QFont TextOverlayWidget::font()
{
	return m_font;
}

void TextOverlayWidget::setFont( const QFont& font )
{
	if ( m_font != font )
	{
		m_font = font;
		m_dirty = true;
		update();
	}
}

void TextOverlayWidget::setBold( bool bold )
{
	if ( m_font.bold() != bold )
	{
		m_font.setBold( bold );
		m_dirty = true;
		update();
	}
}

void TextOverlayWidget::setItalic( bool italic )
{
	if ( m_font.italic() != italic )
	{
		m_font.setItalic( italic );
		m_dirty = true;
		update();
	}
}

void TextOverlayWidget::setUnderline( bool underline )
{
	if ( m_font.underline() != underline )
	{
		m_font.setUnderline( underline );
		m_dirty = true;
		update();
	}
}

void TextOverlayWidget::setStyle( bool bold, bool italic, bool underline )
{
	if ( m_font.bold() != bold || m_font.italic() != italic || m_font.underline() != underline )
	{
		m_font.setBold( bold );
		m_font.setItalic( italic );
		m_font.setUnderline( underline );
		m_dirty = true;
		update();
	}
}

QColor TextOverlayWidget::color()
{
	return m_pen.color();
}

void TextOverlayWidget::setColor( const QColor& color )
{
	if ( m_pen.color() != color )
	{
		m_pen.setColor( color );

		// the translucent color must not be the same as the pen color
		// if it is, we change it
		if ( color == m_transColor )
		{
			int blue = qBlue( m_transRGB );
			m_transColor.setRgb( 0, 0, blue ? blue-1 : blue+1 );
			m_transRGB = qRgb( 0, 0, blue ? blue-1 : blue+1 );
		}

		m_dirty = true;
		update();
	}
}

void TextOverlayWidget::paintEvent( QPaintEvent* e )
{
	if ( m_dirty )
		updateContents();

	QWidget::paintEvent( e );
}

void TextOverlayWidget::resizeEvent( QResizeEvent* )
{
	m_dirty = true;
}

void TextOverlayWidget::updateContents()
{
	QPixmap pixmap( size() );
	QPainter painter( &pixmap );

	painter.setFont( m_font );
	painter.setPen( m_pen );
	painter.fillRect( rect(), m_transColor );
	painter.drawText( rect(), Qt::AlignHCenter|Qt::AlignBottom, m_text );

	if ( ! pixmap.mask() )
		pixmap.setMask( buildMask( pixmap ) );

	setErasePixmap( pixmap );
// 	setBackgroundPixmap( pixmap );
	setMask( *pixmap.mask() );

	m_dirty = false;
}

QBitmap TextOverlayWidget::buildMask( const QPixmap& pixmap )
{
	QImage maskImage( pixmap.size(), 1, 2, QImage::LittleEndian );
	QImage image = pixmap.convertToImage();

	for ( int w = 0; w < width(); ++w )
	{
		for ( int h = 0; h < height(); ++h )
		{
			if ( image.pixel( w, h ) == m_transRGB )
				maskImage.setPixel( w, h, 0 );
			else
				maskImage.setPixel( w, h, 1 );
		}
	}
	QBitmap m; m.convertFromImage( maskImage );
	return m;
}
