/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "timeedit.h"
#include "timevalidator.h"

TimeEdit::TimeEdit( QWidget* parent ):
	KIntSpinBox( parent, 0 ),
	m_validator( new TimeValidator( this ) )
{
	setValidator( m_validator );
	setLineStep( 100 );
	setMinValue( 0 );
	setMaxValue( 86399999 );
}

QSize TimeEdit::sizeHint() const
{
	QSize size = KIntSpinBox::sizeHint();
	size.setWidth( size.width() + 5 );
	return size;
}

QString TimeEdit::mapValueToText( int value )
{
	int hours = value / 3600000;
	int minutes = (value % 3600000) / 60000;
	int seconds = (value % 60000) / 1000;
	int mseconds = value % 1000;

	return QString().sprintf( "%02d:%02d:%02d.%03d", hours, minutes, seconds, mseconds );
}

int TimeEdit::mapTextToValue( bool* ok )
{
	int hours, minutes, seconds, mseconds;
	bool valid = m_validator->parse( cleanText(), hours, minutes, seconds, mseconds );
	if ( ok )
		*ok = valid;
	return valid ? hours * 3600000 + minutes * 60000 + seconds * 1000 + mseconds : 0;
}

bool TimeEdit::eventFilter( QObject* object, QEvent* event )
{
	bool ret = KIntSpinBox::eventFilter( object, event );

	if ( event->type() == QEvent::KeyRelease )
	{
		// special processing for key press
		QKeyEvent* keyEvent = static_cast<QKeyEvent*>( event );
		if ( keyEvent->key() == Qt::Key_Return )
			emit valueEntered( value() );
	}

	return ret;
}
