/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "timevalidator.h"
#include <math.h>

TimeValidator::TimeValidator( QObject* parent ):
	QValidator( parent, 0 ),
	m_validationRegExp( "^([0-9]{0,}):([0-9]{0,}):([0-9]{0,})(\\.[0-9]{0,}|)$" ),
	m_parserRegExp( "^([0-2]?[0-9]?):([0-5]?[0-9]?):([0-5]?[0-9]?)(\\.[0-9]{1,3}|)$" )
{
}

QValidator::State TimeValidator::validate( QString& input, int& ) const
{
	input = input.stripWhiteSpace();
	input.replace( " ", "" );

	if ( m_validationRegExp.search( input ) != -1 )
	{
		int hours = m_validationRegExp.cap( 1 ).toInt();
		int minutes = m_validationRegExp.cap( 2 ).toInt();
		int seconds = m_validationRegExp.cap( 3 ).toInt();
		//int mseconds = m_validationRegExp.cap( 4 ).toInt();

		if ( hours > 23 || minutes > 59 || seconds > 59 )
			return QValidator::Intermediate;
		else
			return QValidator::Acceptable;
	}
	else
		return QValidator::Invalid;
}

bool TimeValidator::parse( const QString& input, int& hours, int& minutes, int&seconds, int& mseconds )
{
	if ( m_parserRegExp.search( input ) != -1 )
	{
		hours = m_parserRegExp.cap( 1 ).toInt();
		minutes = m_parserRegExp.cap( 2 ).toInt();
		seconds = m_parserRegExp.cap( 3 ).toInt();
		QString msecs = m_parserRegExp.cap( 4 ).mid( 1 );
		mseconds = (int)pow( msecs.toInt(), 4 - msecs.length() );
		return true;
	}
	else
		return false;
}

bool TimeValidator::parse( const QString& input, int& timeMillis )
{
	if ( m_parserRegExp.search( input ) != -1 )
	{
		QString msecs = m_parserRegExp.cap( 4 ).mid( 1 );
		timeMillis =	m_parserRegExp.cap( 1 ).toInt() * 3600000 +
						m_parserRegExp.cap( 2 ).toInt() * 60000 +
						m_parserRegExp.cap( 3 ).toInt() * 1000 +
						(int)pow( msecs.toInt(), 4 - msecs.length() );
		return true;
	}
	else
		return false;
}

// void TimeValidator::fixup( QString& input ) const
// {
// 	QValidator::fixup( input );
// }
